/**
 * Replay.java
 *
 */
package client.ds;

import client.CompressedXMLFileReader;
import common.Tuple;
import common.hive.HexDirection;
import common.hive.PieceType;
import common.interfaces.HiveInterface;
import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;

/**
 * A class storing information of a Hive replay.
 *
 * @author Jan-Philipp Kappmeier, Martin Gross
 */
public class Replay {

    private ArrayList<ReplayEntry> entries = null;
    private HiveInterface hiveModel;
    private int replayIndex = -1;

    public Replay(HiveInterface hiveModel) {
        this.hiveModel = hiveModel;
    }

    public int load(CompressedXMLFileReader<LinkedList<String>> a) {
        parseString(a.load());
        return entries.size();
    }

    public boolean replayOneStepForward() {
        if (replayIndex + 1 < entries.size()) {
            ReplayEntry re = entries.get(++replayIndex);
            switch (re.action) {
                case WHITE_FIRST_PIECE:
                    hiveModel.introduce(re.piece);
                    break;
                case BLACK_FIRST_PIECE:
                    hiveModel.introduce(re.piece, re.hexDirection);
                    break;
                case INTRODUCE_PIECE:
                    hiveModel.introduce(re.piece, re.anchor, re.hexDirection);
                    break;
                case MOVE_PIECE:
                    Tuple<Integer, HexDirection> oldAnchor = hiveModel.getAnchor(re.movedIndex);
                    hiveModel.move(re.movedIndex, re.anchor, re.hexDirection);
                    if (oldAnchor == null) {
                        throw new IllegalStateException("oldAnchor is null!");
                    }
                    re.anchor = oldAnchor.getFirst();
                    re.hexDirection = oldAnchor.getSecond();
                    break;
                default:
                    throw new IllegalStateException("Other actions are not supported!");
            }
        } else {
            return false;
        }
        return true;
    }

    public boolean replayOneStepBackward() {
        if (replayIndex >= 0) {
            ReplayEntry re = entries.get(replayIndex--);
            switch (re.action) {
                case WHITE_FIRST_PIECE:
                    hiveModel.revertIntroduction();
                    break;
                case BLACK_FIRST_PIECE:
                    hiveModel.revertIntroduction();
                    break;
                case INTRODUCE_PIECE:
                    hiveModel.revertIntroduction();
                    break;
                case MOVE_PIECE:
                    Tuple<Integer, HexDirection> oldAnchor = hiveModel.getAnchor(re.movedIndex);
                    hiveModel.revertMove(re.movedIndex, re.anchor, re.hexDirection);
                    if (oldAnchor == null) {
                        throw new IllegalStateException("oldAnchor is null!");
                    }
                    re.anchor = oldAnchor.getFirst();
                    re.hexDirection = oldAnchor.getSecond();
                    break;
                default:
                    throw new IllegalStateException("Other actions are not supported!");
            }
        } else {
            return false;
        }
        return true;
    }

    public int size() {
        return entries.size();
    }

    public boolean isAtEnd() {
        return replayIndex == entries.size() - 1;
    }

    public boolean isAtStart() {
        return replayIndex == -1;
    }

    private void parseString(List<String> a) {
        entries = new ArrayList<ReplayEntry>(a.size());
        for (String s : a) {
            ReplayEntry t = new ReplayEntry(s);
            if (t.action != null) {
                entries.add(t);
            }
        }
    }

    private enum HiveActions {

        SET_WHITE_PLAYER,
        WHITE_FIRST_PIECE,
        BLACK_FIRST_PIECE,
        INTRODUCE_PIECE,
        MOVE_PIECE;
    }

    private class ReplayEntry {

        HiveActions action;
        String sender;
        PieceType piece;
        HexDirection hexDirection;
        int anchor;
        int movedIndex = -1;

        private ReplayEntry(HiveActions action, String sender, PieceType piece, HexDirection hexDirection, int anchor) {
            this.action = action;
            this.sender = sender;
            this.piece = piece;
            this.hexDirection = hexDirection;
            this.anchor = anchor;
        }

        private ReplayEntry(String logEntry) {
            String[] p = logEntry.split(", ");
            sender = p[1];
            if (p[2].equals("WHITE_FIRST_PIECE")) {
                action = HiveActions.WHITE_FIRST_PIECE;
                piece = PieceType.values()[Integer.parseInt(p[3])];
            } else if (p[2].equals("BLACK_FIRST_PIECE")) {
                action = HiveActions.BLACK_FIRST_PIECE;
                piece = PieceType.values()[Integer.parseInt(p[3])];
                hexDirection = HexDirection.values()[Integer.parseInt(p[4])];
            } else if (p[2].equals("INTRODUCE_PIECE")) {
                action = HiveActions.INTRODUCE_PIECE;
                piece = PieceType.values()[Integer.parseInt(p[3])];
                anchor = Integer.parseInt(p[4]);
                hexDirection = HexDirection.values()[Integer.parseInt(p[5])];
            } else if (p[2].equals("MOVE_PIECE")) {
                action = HiveActions.MOVE_PIECE;
                movedIndex = Integer.parseInt(p[3]);
                anchor = Integer.parseInt(p[4]);
                hexDirection = HexDirection.values()[Integer.parseInt(p[5])];
            }
        }

        @Override
        public String toString() {
            return sender + ": " + action + ", " + piece + ", " + hexDirection + ", " + anchor + " (" + movedIndex + ")";
        }
    }
}
