/**
 * JBoardPanel.java
 * 
 */
package client.gui;

import client.ds.PieceData;
import common.GameFactory;
import common.Tuple;
import common.hive.HexDirection;
import common.hive.PieceType;
import common.hive.PlayerColor;
import common.interfaces.HexFieldInterface;
import common.interfaces.HiveInterface;
import common.notification.IntroductionNotification;
import common.notification.MoveNotification;
import common.notification.RemoveNotification;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.RenderingHints;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.util.ArrayList;
import java.util.Observable;
import java.util.Observer;
import javax.swing.JOptionPane;
import javax.swing.JPanel;

/**
 * A component representing an array of hexagonal fields. Each of the fields can
 * contain a piece. It reacts to mouse clicks, communicates with a hive model
 * and highlights specific fields. It also is an observer and reacts to
 * automatic messages by an observable. These occur if the model has been
 * changed.
 *
 * @author Jan-Philipp Kappmeier, Martin Gross
 */
@SuppressWarnings( "serial")
public class JBoardPanel extends JPanel implements MouseListener, MouseMotionListener, Observer {

    /**
     * The game model.
     */
    private HiveInterface hiveInterface;
    /**
     * Decides if the game is offline or online.
     */
    private boolean networkGame = false;
    /**
     * The hex array for the hexagonal field.
     */
    private HexFieldInterface<JHexagonTile> hexArray;
    /**
     * The number of possibly reachable hex rows.
     */
    private int rows;
    /**
     * The number of possibly reachable hex columns.
     */
    private int columns;
    /**
     * A list of all pieces that have been placed.
     */
    private PieceData[] pieces = new PieceData[26];
    /**
     * The zoom factor of the board.
     */
    private float zoomFactor = 1;
    /**
     * The index of a selected piece, if any. -1 if no piece is selected.
     */
    private int highlighted = -1;
    /**
     * The piece type that should be introduced if a mouse click on a possible
     * position occurs.
     */
    private PieceType introducePieceType;
    /**
     * Decides if the next click should introduce a piece, or not.
     */
    private boolean introducePiece = false;
    /**
     * A list of fields that are reachable by the currently selected piece.
     */
    private ArrayList<Integer> reachable = new ArrayList<Integer>();
    /**
     * Decides if the board is enabled and reacts to mouse clicks, or not.
     */
    private boolean enabled = true;
    /**
     * The color of the player using this board (in network game)
     */
    private PlayerColor player;
    /**
     * The value of the highest index of a piece introduced so far.
     */
    private int lastIntroduced = -1;
    private JPlayerPanel pieceHeap;
    private boolean drawTileIndices = false;
    private boolean drawEmptyTiles = true;

    /**
     * Sets up a board and initializes the necessary data structures.
     *
     * @param rows the number of hex rows
     * @param columns the number of hex columns
     */
    JBoardPanel(int rows, int columns) {
        //super(null, honey);
        this.rows = rows;
        this.columns = columns;
        setBackground(Color.white);
        initComponent();
        computeSize();
    }

    public boolean isDrawTileIndices() {
        return drawTileIndices;
    }

    public void setDrawTileIndices(boolean drawTileIndices) {
        this.drawTileIndices = drawTileIndices;
    }

    public boolean isDrawEmptyTiles() {
        return drawEmptyTiles;
    }

    public void setDrawEmptyTiles(boolean drawEmptyTiles) {
        this.drawEmptyTiles = drawEmptyTiles;
    }

    /**
     * Resets the component, so that a new game can be started.
     */
    public void reset() {
        computeCoordinates();
        //	/** The game model. */
        hiveInterface = null;
//	/** A list of all pieces that have been placed. */
        pieces = new PieceData[26];
//	/** The index of a selected piece, if any. -1 if no piece is selected. */
        highlighted = -1;
//	/** The piece type that should be introduced if a mouse click on a possible position occurs. */
        introducePieceType = null;
//	/** Decides if the next click should introduce a piece, or not. */
        introducePiece = false;
//	/** A list of fields that are reachable by the currently selected piece. */
        reachable = new ArrayList<Integer>();
//	/** The value of the highest index of a piece introduced so far. */
        lastIntroduced = -1;
        repaint();
    }

    /**
     * Sets the preferred and actual size of the board, depending on the size of
     * the hex fields.
     */
    private void computeSize() {
        final Dimension d = new Dimension((int) (hexArray.getWidth() * (JHexagonTile.getWidth() - JHexagonTile.getHalf()) + JHexagonTile.getHalf() + 1), (int) (hexArray.getHeight() * (JHexagonTile.getHeight()) + JHexagonTile.getHeight() * 0.5 + 1));
        setSize(d);
        setPreferredSize(d);
    }

    /**
     * Sets up the data structure for the hexagonal array.
     */
    private void initComponent() {
        hexArray = GameFactory.createNewHexField(rows, columns, JHexagonTile.class);
        //computeCoordinates();
        addMouseListener(this);
        addMouseMotionListener(this);
    }

    /**
     * Computes an index number for each hexagonal field.
     */
    private void computeCoordinates() {

        // Init the hex fields
        int cnt = 0;
        for (int i = 0; i < rows; ++i) {
            for (int j = 0; j < columns; ++j) {
                JHexagonTile c = new JHexagonTile(cnt++);
                // move every second half height down
                c.setPosition(j * (JHexagonTile.getWidth() - JHexagonTile.getHalf()), i * JHexagonTile.getHeight() + (j & 1) * (JHexagonTile.getHeight() * 0.5));
                hexArray.set(i, j, c);
            }
        }
    }

    /**
     * Paints the component, that is, it clears the background and calls the
     * paint method for all the sub elements (the piece).
     *
     * @param g the graphics context
     */
    @Override
    public void paintComponent(Graphics g) {
        super.paintComponent(g);

        Graphics2D g2 = (Graphics2D) g;
        g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);

        //Clear background - do this only once - the polygons and edges won't do it.
        g2.setPaint(getBackground());
        g2.fillRect(0, 0, getWidth(), getHeight());

        // draw components
        for (JHexagonTile piece : hexArray) {
            if (piece != null) {
                if (drawEmptyTiles) {
                    piece.drawBorder(g2);
                }
                piece.paintPiece(g2);
                if (drawTileIndices) {
                    piece.drawText(g2);
                }
            }
        }
    }

    /**
     * <p>Marks a hexagonal field with the specified index. A previously marked
     * hex field is unmarked. An index of -1 clears the mark.</p> <p>If the
     * current player can move the marked piece, the reachable fields are
     * received from the game model and the reachable fields are highlighted as
     * well.</p>
     *
     * @param hexIndex the hex field
     */
    private void mark(int hexIndex) {
        // first clear
        if (highlighted >= 0) {
            hexArray.get(highlighted).setHighligted(false);
        }
        for (int i : reachable) {
            hexArray.get(i).setReachable(false);
        }
        reachable.clear();

        if (hexIndex == -1) {
            highlighted = -1;
            return;
        }

        // return if the hitted piece is not from the current player
        if (hiveInterface.getCurrentPlayer() == null) {
            throw new NullPointerException("getCurrentPlayer returned null");
        }
        if (hiveInterface.getCurrentPlayer() != hexArray.get(hexIndex).piece.getColor()) {
            return;
        }

        ArrayList<Tuple<Integer, HexDirection>> newReachable = hiveInterface.getReachable(hexArray.get(hexIndex).piece.getIndex());
        if (newReachable == null) {
            throw new NullPointerException("getReachable returned null");
        }

        hexArray.get(hexIndex).setHighligted(true);
        highlighted = hexIndex;
        for (Tuple<Integer, HexDirection> t : newReachable) {
            hexArray.get(hexArray.getNeighborIndex(pieces[t.getFirst()].piecePosition, t.getSecond())).setReachable(true);
            reachable.add(hexArray.getNeighborIndex(pieces[t.getFirst()].piecePosition, t.getSecond()));
        }
    }

    /**
     * For a given index, an adjacent piece is searched. This is done because
     * the model can only handle relative coordinates to pieces already placed.
     *
     * @param index the index of the piece that is addressed
     * @return a tuple containing the index of an adjacent piece and a direction
     * in which the searched piece lies
     * @throws IllegalStateException if the given piece has no occupied neighbor
     */
    private Tuple<Integer, HexDirection> findAdjacentPiece(int index) throws IllegalStateException {
        for (HexDirection dir : HexDirection.values()) {
            if (hexArray.get(hexArray.getNeighborIndex(index, dir)).piece != null && (hexArray.getNeighborIndex(index, dir) != highlighted || hexArray.get(highlighted).piece.onTop)) {
                return new Tuple<Integer, HexDirection>(hexArray.getNeighborIndex(index, dir), dir.opposite());
            }
        }
        throw new IllegalStateException("JBoard.java: Reachable field has no occupied neighbor!");
    }

    /**
     * A method that highlights fields that are available for insertion of new
     * pieces.
     *
     * @param pieceType the piece type that should be inserted with the next
     * click
     */
    void highlightInsertionFields(PieceType pieceType) {
        ArrayList<Tuple<Integer, HexDirection>> introduceable = hiveInterface.getIntroduceable();
        if (introduceable == null) {
            throw new NullPointerException("getIntroduceable returned null");
        }

        for (int i : reachable) {
            hexArray.get(i).setReachable(false);
        }
        reachable.clear();

        for (Tuple<Integer, HexDirection> t : introduceable) {
            hexArray.get(hexArray.getNeighborIndex(pieces[t.getFirst()].piecePosition, t.getSecond())).setReachable(true);
            reachable.add(hexArray.getNeighborIndex(pieces[t.getFirst()].piecePosition, t.getSecond()));
        }
        introducePiece = true;
        introducePieceType = pieceType;
    }

    /**
     * Introduces the first figure to the field.
     *
     * @param pieceType the piece type that is introduced.
     */
    void introduce(PieceType pieceType) {
        hiveInterface.introduce(pieceType);
        repaint();
    }

    /**
     * The update callback method for the observer. If it is called, the latest
     * model update is received from the {@code notification} and the view is
     * updated.
     *
     * @param observable the observable object. ignored.
     * @param notification the content of the last move. must be of type
     * {@link PieceNotification}
     * @see PieceNotification
     */
    public void update(Observable observable, Object notification) {
        int targetPos = -1;
        if (notification instanceof IntroductionNotification) {
            IntroductionNotification intro = (IntroductionNotification) notification;

            if (intro.getPieceIndex() != lastIntroduced + 1 || intro.getPieceIndex() < 0) {
                throw new IllegalArgumentException("The piece index is wrong: was " + intro.getPieceIndex());
            }
            if (hiveInterface.getPieceType(intro.getPieceIndex()) == null) {
                throw new IllegalStateException("getPieceType returned null");
            }

            if (intro.getDirection() == null) {
                if (lastIntroduced != -1) {
                    throw new IllegalStateException("Introduction notification for fist Piece called more than once!");
                }

                targetPos = hexArray.getCenter();

                PieceData cd = new PieceData(hiveInterface.getPieceType(intro.getPieceIndex()), intro.getColor(), intro.getPieceIndex());
                cd.piecePosition = targetPos;
                hexArray.get(targetPos).piece = cd;
                pieces[intro.getPieceIndex()] = cd;
                // add to the center
                Rectangle bounds = new Rectangle(hexArray.get(targetPos).getBounds());
                bounds.setLocation(bounds.x - getVisibleRect().width / 2, bounds.y - getVisibleRect().height / 2);
                bounds.setSize(bounds.width + getVisibleRect().width, bounds.height + getVisibleRect().height);
                scrollRectToVisible(bounds);
                lastIntroduced++;
            } else {
                
                // introduce another
                PieceData cd = new PieceData(hiveInterface.getPieceType(intro.getPieceIndex()), intro.getColor(), intro.getPieceIndex());
                targetPos = hexArray.getNeighborIndex(pieces[intro.getAnchorIndex()].piecePosition, intro.getDirection());
                cd.piecePosition = targetPos;
                pieces[intro.getPieceIndex()] = cd;
                hexArray.get(targetPos).piece = cd;
                lastIntroduced++;
            }
        } else if (notification instanceof RemoveNotification) {
            RemoveNotification remove = (RemoveNotification) notification;
            int hexPos = pieces[remove.getPieceIndex()].piecePosition;
            hexArray.get(hexPos).piece = null;
            lastIntroduced--;

        } else if (notification instanceof MoveNotification) {
            MoveNotification move = (MoveNotification) notification;

            if (move.getAnchorIndex() > lastIntroduced || move.getAnchorIndex() < 0) {
                throw new IllegalArgumentException("Error during move. anchorIndex of notification was wrong: " + move.getAnchorIndex() + " Must be in the range of 0 to " + lastIntroduced);
            }
            if (move.getPieceIndex() > lastIntroduced || move.getPieceIndex() < 0) {
                throw new IllegalArgumentException("Error during move. pieceIndex of notification was wrong: " + move.getPieceIndex() + " Must be in the range of 0 to " + lastIntroduced);
            }
            if (hiveInterface.getPieceType(move.getPieceIndex()) == null) {
                throw new IllegalStateException("getPieceType returned null");
            }
            if (move.getDirection() == null) {
                throw new NullPointerException("MoveNotification returned null for direction");
            }
            if (move.getBelow() < -1 || move.getBelow() > lastIntroduced) {
                throw new IllegalArgumentException("Error during move. below of notification was wrong: " + move.getBelow() + " Must be in the range of -1 to " + lastIntroduced);
            }

            int startPos = pieces[move.getPieceIndex()].piecePosition;
            targetPos = hexArray.getNeighborIndex(pieces[move.getAnchorIndex()].piecePosition, move.getDirection());

            pieces[move.getPieceIndex()].onTop = hexArray.get(targetPos).piece != null;
            pieces[move.getPieceIndex()].piecePosition = targetPos;
            hexArray.get(targetPos).piece = pieces[move.getPieceIndex()];
            hexArray.get(startPos).piece = move.getBelow() != -1 ? pieces[move.getBelow()] : null;
        }

        // here a new piece may have been added to near to the border.
        if (targetPos >= 0 && hexArray.checkDangerousPosition(targetPos) != 0) // has been recentered!
        {
            for (JHexagonTile c : hexArray) {
                if (c.containsData()) {
                    pieces[c.piece.getIndex()].piecePosition = c.getIndex();
                }
            }
        }

        if (networkGame) {
            setElementsEnabled(player == hiveInterface.getCurrentPlayer());
        }

        repaint();
    }

    /**
     * Marks fields if the current player is active. If a piece is to be
     * introduced, a click will introduce it. Otherwise, if a piece was marked,
     * and a reachable field is hit by the click, the piece is moved to that
     * position.
     *
     * @param e
     */
    public void mouseClicked(MouseEvent e) {
        if (hiveInterface == null || hiveInterface.isGameOver() || !enabled) {
            return;
        }
        // reset height
        // check, if a field was hit
        int hit = -1;
        for (JHexagonTile c : hexArray) {
            if (c.inside(new Point(e.getX(), e.getY()))) {
                hit = c.getIndex();
            }
        }

        // check for the mode where a piece should be introduced
        if (introducePiece) {
            // check if one of the highlighted fields was hit, in that case add the
            // new figure to that place
            if (hexArray.get(hit).isReachable()) {
                // introduce the element to the highlighted place

                // find a neighbor
                for (HexDirection dir : HexDirection.values()) {
                    JHexagonTile c = hexArray.get(hexArray.getNeighborIndex(hit, dir));
                    if (c.piece != null) {
                        if (hiveInterface.getIntroducedPieces() != 1) {
                            // other pieces
                            try {
                                hiveInterface.introduce(introducePieceType, c.piece.getIndex(), dir.opposite());
                            } catch (Exception ex) {
                                JOptionPane.showMessageDialog(this, "Fehler beim Aufruf von introduce():" + ex.getMessage(), "Fehler", JOptionPane.ERROR_MESSAGE);
                                ex.printStackTrace();
                            }
                        } else // the first black piece
                        {
                            try {
                                hiveInterface.introduce(introducePieceType, dir.opposite());
                            } catch (Exception ex) {
                                JOptionPane.showMessageDialog(this, "Fehler beim Aufruf von introduce(): " + ex.getMessage(), "Fehler", JOptionPane.ERROR_MESSAGE);
                                ex.printStackTrace();
                            }
                        }
                        for (int i : reachable) {
                            hexArray.get(i).setReachable(false);
                        }
                        reachable.clear();
                        break;
                    }
                }
                mark(-1);
                repaint();
                return;
            } else {
                pieceHeap.clearSelection();
            }
            introducePiece = false;
        }

        // check, if a marked field was hit. in this case, move
        if (hit >= 0 && reachable.contains(hit)) {
            Tuple<Integer, HexDirection> index = findAdjacentPiece(hit);
            try {
                hiveInterface.move(hexArray.get(highlighted).piece.getIndex(), hexArray.get(index.getFirst()).piece.getIndex(), index.getSecond());
            } catch (Exception ex) {
                JOptionPane.showMessageDialog(this, "Fehler beim Aufruf von move():" + ex.getMessage(), "Fehler", JOptionPane.ERROR_MESSAGE);
                ex.printStackTrace(System.out);
            }
            mark(-1);
        } else {
            mark(hit >= 0 && hexArray.get(hit).piece != null ? hit : -1);
        }

        // check, if a field was hit
        repaint();
    }

    /**
     * Not implemented mouse method.
     *
     * @param e the mouse event
     */
    public void mousePressed(MouseEvent e) {
    }

    /**
     * Not implemented mouse method.
     *
     * @param e the mouse event
     */
    public void mouseReleased(MouseEvent e) {
    }

    /**
     * Not implemented mouse method.
     *
     * @param e the mouse event
     */
    public void mouseEntered(MouseEvent e) {
    }

    /**
     * Not implemented mouse method.
     *
     * @param e the mouse event
     */
    public void mouseExited(MouseEvent e) {
    }

    /**
     * Sets the status for the online/offline version.
     *
     * @param player the color of the starting player
     * @param networkGame decides if the game is online or offline
     */
    void setNetworkGame(PlayerColor player, boolean networkGame) {
        this.networkGame = networkGame;
        this.player = player;
        setElementsEnabled(player == PlayerColor.WHITE);
        computeCoordinates();
    }

    /**
     * Sets the model that is viewed by this component.
     *
     * @param hiveInterface the game model
     */
    void setHiveInterface(HiveInterface hiveInterface) {
        this.hiveInterface = hiveInterface;
    }

    /**
     * Disables and enables the game board.
     *
     * @param value decides if the board is enabled or disabled
     */
    void setElementsEnabled(boolean value) {
        enabled = value;
        if (!value) {
            if (highlighted >= 0) {
                hexArray.get(highlighted).setHighligted(false);
            }
            highlighted = -1;
            for (int i : reachable) {
                hexArray.get(i).setReachable(false);
            }
            reachable.clear();
        }
        repaint();
    }

    /**
     * Zooms the game board to the original size times a given factor.
     *
     * @param factor the zoom factor
     */
    void zoom(double factor) {
        JHexagonTile.computeParameters(factor * 70);
        for (int i = 0; i < rows; ++i) {
            for (int j = 0; j < columns; ++j) {
                final double x = j * (JHexagonTile.getWidth() - JHexagonTile.getHalf());
                final double y = i * JHexagonTile.getHeight() + (j & 1) * (JHexagonTile.getHeight() * 0.5);// move every second half height down
                hexArray.get(i, j).setPosition(x, y);
            }
        }
        computeSize();
        repaint();
    }

    public void mouseDragged(MouseEvent e) {
    }

    public void mouseMoved(MouseEvent e) {
        final JHexagonTile hti = getHexTileIndex(e.getX(), e.getY());
        if (hti != null && hti.piece != null) {
            setToolTipText("<html>Hex-Feld " + hti.getIndex() + "<br>Besetzt von Stein " + hti.piece.getIndex() + "</html>");
        } else {
            setToolTipText("");
        }
    }

    private JHexagonTile getHexTileIndex(int mouseX, int mouseY) {
        // constants in the hex tile
        final double r = JHexagonTile.getHeight() / 2;
        final double h = JHexagonTile.getHalf();
        final double s = JHexagonTile.getWidth() - 2 * JHexagonTile.getHalf();
        // the x and y coordinates in an hypothetical array of squares
        final int xSection = (int) Math.floor(mouseX / (s + h));
        final int ySection = (int) Math.floor(mouseY / (2 * r));
        // the offsets within the hypothetical array of squares
        final int xOffset = (int) (mouseX - xSection * (h + s));
        final int yOffset = (int) (mouseY - ySection * (2 * r));
        // the slope of the top-left (and bottom right) side
        final double m = r / h;

        // basic computation of array position
        int arrayX = xSection;
        int arrayY = ySection;

        // unfortunateley hex tiles are not squares, so they overlap the squares
        // indicated by xSect and ySect. So we have to adjust in some cases...
        // compute section, depends if they are odd or even
        if ((xSection & 1) == 0) { // Type = A
            if (yOffset < r - m * xOffset) { // corner top left
                arrayX--;
                arrayY--;
            } else if (yOffset > (r + m * xOffset)) { // corner at bottom left
                arrayX--;
            } // else Big rectangle part
        } else { // Type = B
            if ((yOffset > m * xOffset) && (yOffset < 2 * r - m * xOffset)) // triangle left
            {
                arrayX--;
            } else if ((yOffset < m * xOffset) && yOffset < r) // Upper part
            {
                arrayY--; // else lower part
            }
        }
        return arrayX >= 0 && arrayY >= 0 && arrayX < hexArray.getWidth() && arrayY < hexArray.getHeight() ? hexArray.get(arrayY, arrayX) : null;
    }

    void setPieceHeap(JPlayerPanel pieceHeap) {
        this.pieceHeap = pieceHeap;
    }
}