/**
 * JHexagonTile.java
 * 
 */
package client.gui;

import client.ds.PieceData;
import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.FontMetrics;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.RenderingHints;
import common.interfaces.DataContainer;
import java.awt.Font;
import java.awt.image.BufferedImage;
import common.hive.PlayerColor;

/**
 * Graphical representation of a piece in hive. It can display a background
 * color (indicating black or white player), an image for the type of piece and
 * a hexagonal border. It stores some information about its position and
 * provides static data for the size of a hexagonal field. But, it does not
 * extend a {@link JComponent}.
 *
 * @author Jan-Philipp Kappmeier
 */
@SuppressWarnings( "serial")
public class JHexagonTile implements DataContainer<PieceData> {

    /**
     * The number of the hexagonal field, the order starts in the top-left
     * corner and ends at the bottom-right.
     */
    private int hexID;
    /**
     * A dashed stroke used to draw the border of empty fields.
     */
    private final static BasicStroke dashed = new BasicStroke(1.0f, BasicStroke.CAP_BUTT, BasicStroke.JOIN_MITER, 10.0f, new float[]{5.0f, 20}, 0.0f);
    /**
     * A thick stroke used for highlighted fields.
     */
    private final static BasicStroke thick = new BasicStroke(3);
    /**
     * The default stroke.
     */
    private final static BasicStroke normal = new BasicStroke(1);
    /**
     * Possible background colors, depending on the player and the highlight
     * status.
     */
    private static final Color[] BACKGROUND = {Color.white, Color.black, Color.yellow};
    /**
     * Possible border colors for empty, reachable and default fields.
     */
    private static final Color[] BORDER = {Color.RED, Color.GREEN, Color.BLACK};
    /**
     * The height of a hexagonal field.
     */
    private static double height = 70;
    /**
     * The width of a hexagonal field.
     */
    private static double width = 80.82903768654761;
    /**
     * The half of the length of the radius (length of the center-corner line).
     */
    private static double half = 20.2072594216369;
    /**
     * The x-coordinate of the center of the hexagonal field.
     */
    private double x;
    /**
     * The y-coordinate of the center of the hexagonal field.
     */
    private double y;
    /**
     * The x-coordinates of the corners.
     */
    private int xPoints[] = new int[6];
    /**
     * The y-coordinates of the corners.
     */
    private int yPoints[] = new int[6];
    /**
     * Decides if a field is reachable. In that case its border is drawn
     * differently.
     */
    private boolean reachable;
    /**
     * Decides if a field is highlighted. In that case the color is drawn
     * differently.
     */
    private boolean highligted;
    /**
     * Decides if the index of a hex field should be drawn, or not.
     */
    private static boolean drawIndex = true;
    /**
     * Decides if the border of an empty hex field should be drawn, or not.
     */
    static boolean drawBorder = false;
    /**
     * A bounding box for the hex field. Used to draw text inside.
     */
    private Rectangle bounds = new Rectangle((int) x, (int) y, (int) width + 1, (int) height + 1);
    /**
     * Data for the pieces inside.
     */
    PieceData piece;

    /**
     * Initializes the hexagonal field with a given id.
     *
     * @param hexID
     */
    public JHexagonTile(int hexID) {
        super();
        this.hexID = hexID;
    }

    /**
     * Computes values for a piece that are used to display. This includes the
     * width and height of the component (that is necessary to draw a hexagonal
     * on it) and computes coordinates of the corners of the hexagon. Basic unit
     * is the height of the component, the other values are relative to the
     * height.
     *
     * @param height the height of a hexagonal field. other parameters follow
     * from the height
     */
    public static void computeParameters(double height) {
        JHexagonTile.height = height;
        //JPiece.height = height;
        width = height / Math.cos(Math.PI / 6);
        half = Math.sqrt(1. / 12) * height;
    }

    /**
     * Sets the position for this instance of a hexagonal field.
     *
     * @param x the x-coordinate of the center
     * @param y the y-coordinate of the center
     */
    public void setPosition(double x, double y) {
        this.y = y;
        this.x = x;
        // Set up corner coordinates
        xPoints[0] = (int) Math.round(x + width * 0.5 - half);
        xPoints[1] = (int) Math.round(x + width * 0.5 + half);
        xPoints[2] = (int) Math.round(x + width);
        xPoints[3] = (int) Math.round(x + width * 0.5 + half);
        xPoints[4] = (int) Math.round(x + width * 0.5 - half);
        xPoints[5] = (int) Math.round(x + 0);
        yPoints[0] = (int) Math.round(y + 0);
        yPoints[1] = (int) Math.round(y + 0);
        yPoints[2] = (int) Math.round(y + height * 0.5);
        yPoints[3] = (int) Math.round(y + height);
        yPoints[4] = (int) Math.round(y + height);
        yPoints[5] = (int) Math.round(y + height * 0.5);
        bounds = new Rectangle((int) x, (int) y, (int) width + 1, (int) height + 1);
    }

    /**
     * Computes the orientation between three points. The values can be -1,0 or
     * 1. The geometric background the following: two lines starting in point q
     * going to q and r. The question is, if the point r is left of the pq-line
     * or right of it. If the three points lie on a line, the result is 0. Other
     * wise it is -1 or 1.
     *
     * @param p the first point
     * @param q the second point
     * @param r the third point
     * @return the orientation of the three points in {-1,0,1}
     */
    private static int orientation(Point p, Point q, Point r) {
        final double u1 = p.getX() - q.getX();
        final double u2 = p.getY() - q.getY();
        final double v1 = r.getX() - q.getX();
        final double v2 = r.getY() - q.getY();
        final double det = v1 * u2 - v2 * u1;
        return (int) Math.signum(det);
    }

    /**
     * Checks whether a point is inside the hexagonal area, or not. This is an
     * accurate test that does not rely on the bounding box.
     *
     * @param point the point that is tested
     * @return {@code true} if the point is inside the hexagon, {@code false}
     * otherwise
     */
    boolean inside(Point point) {
        // check if in the hexagon
        int count = 0;
        for (int i = 0; i < 6; ++i) {
            final Point p1 = new Point(xPoints[i], yPoints[i]);
            final Point p2 = new Point(xPoints[(i + 1) % 6], yPoints[(i + 1) % 6]);
            if (orientation(p1, p2, point) == 1) {
                count++;
            } else if (orientation(p1, p2, point) == -1) {
                // outside
            } else // border
            {
                count++;
            }
        }
        return count == 6;
    }

    /**
     * Draws the piece that means, it draws the hexagonal border and an image
     * corresponding to the type of piece.
     *
     * @param g2d the graphics object on which the piece is drawn
     */
    public void paintPiece(Graphics2D g2d) {
        try {
            g2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
            g2d.setRenderingHint(RenderingHints.KEY_INTERPOLATION, RenderingHints.VALUE_INTERPOLATION_BICUBIC);
            if (piece != null) {
                if (highligted) {
                    g2d.drawImage(Images.HEX_BIG[2], (int) Math.round(x), (int) Math.round(y), (int) Math.round(width), (int) Math.round(height), null);
                } else if (reachable) {
                    g2d.drawImage(Images.HEX_BIG[3], (int) Math.round(x), (int) Math.round(y), (int) Math.round(width), (int) Math.round(height), null);
                } else if (piece.getColor() == PlayerColor.WHITE) {
                    g2d.drawImage(Images.HEX_BIG[0], (int) Math.round(x), (int) Math.round(y), (int) Math.round(width), (int) Math.round(height), null);
                } else {
                    g2d.drawImage(Images.HEX_BIG[1], (int) Math.round(x), (int) Math.round(y), (int) Math.round(width), (int) Math.round(height), null);
                }
                BufferedImage image = Images.getImageFor(piece.getPieceType(), piece.getColor());
                g2d.drawImage(image, (int) (x + width * 0.15), (int) (y + height * 0.15), (int) (width * 0.7), (int) (height * 0.7), null);
            } else if (reachable) {
                g2d.drawImage(Images.HEX_BIG[3], (int) Math.round(x), (int) Math.round(y), (int) Math.round(width), (int) Math.round(height), null);
            }
        } catch (java.lang.ExceptionInInitializerError ex) {
            throw new ExceptionInInitializerError("Images not correctly loaded");
        }
    }

    public void drawText(Graphics2D g2d) {
        drawText(Integer.toString(hexID), g2d);
    }

    public void drawBorder(Graphics2D g2d) {
        g2d.setColor(Color.BLACK);
        g2d.drawPolygon(xPoints, yPoints, 6);
    }

    /**
     * Draws the specified text at the center of the component. The position is
     * the middle of the bounding box. The text is not displayed if it is to
     * large to fit into the bounding box.
     *
     * @param name the text displayed.
     * @param g2 the graphics context on which the name is drawn
     */
    private void drawText(String name, Graphics2D g2) {
        Font font = g2.getFont();
        Font newFont = font.deriveFont(Font.BOLD, 16.0f);
        g2.setFont(newFont);
        FontMetrics metrics = g2.getFontMetrics();
        Rectangle nameBounds = metrics.getStringBounds(name, g2).getBounds();
        Rectangle myBounds = bounds;
        Rectangle stringBounds = new Rectangle((int) (x + (myBounds.width - nameBounds.width) / 2), (int) (y + (myBounds.height - nameBounds.height) / 2), nameBounds.width, nameBounds.height);
        Rectangle normalizedBounds = new Rectangle((int) x, (int) y, myBounds.width, myBounds.height);
        // Only paint if there is enough screen space for the string. We check it with the
        // correct height values. But the nameBounds are baseline-relative so we need to
        // subtract the negative y-offset for the baseline.
        if (normalizedBounds.contains(stringBounds)) {
            g2.setColor(Color.RED);
            g2.drawString(name, stringBounds.x, stringBounds.y - nameBounds.y);
        }
        g2.setFont(font);
    }

    /**
     * Returns a value that equals half of the radius of the hexagon (the radius
     * is the distance between the center and a corner point). It also equals
     * the width of the triangle shaped left-over parts on the sides of the
     * hexagon.
     *
     * @return half of the radius
     */
    public static double getHalf() {
        return half;
    }

    /**
     * Returns the height of the hexagonal fields.
     *
     * @return the height of the hexagonal fields
     */
    public static double getHeight() {
        return height;
    }

    /**
     * Returns the width of the hexagonal fields.
     *
     * @return the width of the hexagonal fields
     */
    public static double getWidth() {
        return width;
    }

    /**
     * Returns the bounding box.
     *
     * @return
     */
    public Rectangle getBounds() {
        return bounds;
    }

    /**
     * Decides whether this hexagonal field is highlighted. This occurs if a
     * user clicks on it. In that case the background color of the piece
     * changes.
     *
     * @param highlighted the highlight status
     */
    public void setHighligted(boolean highlighted) {
        this.highligted = highlighted;
    }

    /**
     * Decides if if the piece is highlighted, or not.
     *
     * @return {@code true} if the piece is highlighted, {@code false} otherwise
     */
    boolean isHighlighted() {
        return highligted;
    }

    /**
     * Decides whether this hexagonal field is reachable, or not. Reachable
     * fields are highlighted using a different border.
     *
     * @param reachable the reachable status
     */
    public void setReachable(boolean reachable) {
        this.reachable = reachable;
    }

    /**
     * Decides if the piece is reachable, or not.
     *
     * @return {@code true} if the piece is highlighted, {@code false} otherwise
     */
    boolean isReachable() {
        return reachable;
    }

    @Override
    public boolean containsData() {
        return piece != null;
    }

    @Override
    public int getIndex() {
        return hexID;
    }

    /**
     * Moves the data stored in the container to another container.
     *
     * @param t the data container
     * @throws IllegalArgumentException if the parameter {@code t} is not of the
     * correct type
     */
    @Override
    public void moveDataTo(DataContainer<PieceData> t) throws IllegalArgumentException {
        if (!(t instanceof JHexagonTile)) {
            throw new IllegalArgumentException("Can only move Data of type JPiece.");
        }
        moveDataTo((JHexagonTile) t);
    }

    /**
     * Simply overrides the data in the target field. This is a convenience
     * method with appropriate parameter type.
     *
     * @param t the target piece
     */
    public void moveDataTo(JHexagonTile t) {
        t.piece = piece;
    }

    public void resetData() {
        piece = null;
    }

    public PieceData getData() {
        return piece;
    }

    public void setNewData(PieceData piece) {
        this.piece = piece;
    }
}
