/**
 * JHiveFrame.java
 * 
 */
package client.gui;

import client.CompressedXMLFileReader;
import client.ds.Replay;
import client.network.Failed;
import client.network.ServerCommand;
import client.network.ServerHandler;
import client.network.ServerListener;
import client.network.Success;
import common.GameFactory;
import common.hive.HexDirection;
import common.hive.PieceType;
import common.hive.PlayerColor;
import common.interfaces.ClientHiveInterface;
import common.interfaces.HiveInterface;
import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.HeadlessException;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileDescriptor;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.nio.channels.Channels;
import java.util.LinkedList;
import java.util.Observable;
import java.util.Observer;
import javax.swing.DefaultListModel;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.filechooser.FileFilter;

/**
 * The main window of the Hive client.
 *
 * @author Jan-Philipp Kappmeier, Martin Gross
 */
@SuppressWarnings("serial")
public class JHiveFrame extends JFrame implements Observer {

    /**
     * The name of the user (used to connect to the server)
     */
    private String name;
    /**
     * The password used to connect to the server.
     */
    private String password;
    /**
     * The server handler (running in a different thread).
     */
    private ServerHandler handler;
    /**
     * The object used to send commands to the server.
     */
    protected ServerCommand command;
    /**
     * A hive model interface implementation that is connected to a server.
     */
    protected ClientHiveInterface clientHiveModel;
    /**
     * A list box containing all games that are started.
     */
    private DefaultListModel gameList = new DefaultListModel();
    /**
     * A graphical component to start, create or join a game.
     */
    private JStartGameDialog startGameDialog;
    private JStatusBar statusBar;
    /**
     * A hive model implementation used for offline gaming.
     */
    protected HiveInterface hiveModel;
    /**
     * A graphical representation of a hive board (in the upper half of the
     * window).
     */
    protected JHivePanel upper;
    /**
     * The initial zoom factor.
     */
    private int zoom = 0;
    /**
     * The tool bar.
     */
    protected JHiveToolBar toolBar;
    /**
     * A dialog to choose a replay file.
     */
    private JFileChooser replayFileChooser;

    { // static initializer for the file selection dialogs
        replayFileChooser = new JFileChooser(new File("./replay"));
        replayFileChooser.setFileFilter(new FileFilter() {
            @Override
            public boolean accept(File f) {
                return f.isDirectory() || f.getName().toLowerCase().endsWith(".hive");
            }

            @Override
            public String getDescription() {
                return "Hive-Replays";
            }
        });
        replayFileChooser.setAcceptAllFileFilterUsed(false);
    }
    /**
     * A replay of a game loaded from file.
     */
    protected Replay replay;
    private HiveServerListener serverListener;

    /**
     * The Hive main window is displayed with size 800x600 in the center of the
     * screen with the given title.
     *
     * @param title the window title
     * @throws HeadlessException if {@code GraphicsEnvironment.isHeadless()}
     * returns {@code true}
     */
    public JHiveFrame(String title) throws HeadlessException {
        super(title);
        setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        setSize(800, 600);
        Dimension d = Toolkit.getDefaultToolkit().getScreenSize();
        setLocation((d.width - getSize().width) / 2, (d.height - getSize().height) / 2);
        initComponents();
        addWindowListener(windowListener);
    }

    /**
     * Initializes the components on the main window.
     */
    private void initComponents() {
        setLayout(new BorderLayout());
        // Reset the toolbar
        Buttons[] buttonList = {Buttons.StartOfflineGame, null, Buttons.Connect, Buttons.StartOnlineGame, null, Buttons.ZoomOut, Buttons.ZoomIn, Buttons.HexGrid, Buttons.HexNumbers, null, Buttons.LoadReplay, Buttons.ToStart, Buttons.Backward, Buttons.Forward, Buttons.ToEnd, null, Buttons.Exit};
        toolBar = new JHiveToolBar();
        toolBar.setAcl(new ToolBarActionListener());
        toolBar.setButtons(buttonList);
        toolBar.initialize();
        Buttons.Connect.setSelectedIcon(Images.DISCONNECT);
        toolBar.setRollover(true);
        add(toolBar, BorderLayout.NORTH);
        upper = new JHivePanel();
        add(upper, BorderLayout.CENTER);
        statusBar = new JStatusBar("status");
        add(statusBar, BorderLayout.SOUTH);
    }

    protected HiveInterface createNewHiveModel() {
        hiveModel = GameFactory.createNewHiveModel();
        hiveModel.addObserver(this);
        return hiveModel;
    }

    protected void connect(String server, String port, String login, String password) {
        serverListener = new HiveServerListener();
        handler = new ServerHandler(port, server, serverListener, true);
        handler.start();
        name = login;
        this.password = password;
        Buttons.Connect.setSelected(true);
    }

    protected void addWindowListener() {
        addWindowListener(windowListener);
    }
    /**
     * A window listener that performs some necessary operations when the window
     * is shown or closed.
     */
    private WindowListener windowListener = new WindowAdapter() {
        /**
         * Closes the connection to the server when the window is closed.
         */
        @Override
        public void windowClosing(WindowEvent e) {
            // if connected, logout
            if (command != null) {
                command.logout();
            }

        }
    };

    @Override
    public void update(Observable o, Object arg) {
        if (hiveModel.isGameOver()) {
            /*
             if (clientHiveModel != null) {
             Buttons.StartOnlineGame.setEnabled(true);
             }
             if (command == null) {
             Buttons.Connect.setEnabled(true);
             }*/
        }
    }

    /**
     * Enables and disables the replay buttons, according to the replay
     * position.
     */
    private void updateButtons() {
        Buttons.HexGrid.setEnabled(clientHiveModel != null || hiveModel != null || replay != null);
        Buttons.HexNumbers.setEnabled(clientHiveModel != null || hiveModel != null || replay != null);
        Buttons.ZoomIn.setEnabled(clientHiveModel != null || hiveModel != null || replay != null);
        Buttons.ZoomOut.setEnabled(clientHiveModel != null || hiveModel != null || replay != null);
        Buttons.Forward.setEnabled(replay != null && !replay.isAtEnd());
        Buttons.ToEnd.setEnabled(replay != null && !replay.isAtEnd());
        Buttons.Backward.setEnabled(replay != null && !replay.isAtStart());
        Buttons.ToStart.setEnabled(replay != null && !replay.isAtStart());
    }

    public static class InputThread extends Thread {

        boolean running = true;
        PrintWriter o;

        public InputThread(PrintWriter o) {
            super();
            this.o = o;
        }

        @Override
        public void run() {
            BufferedReader in = new BufferedReader(
                    new InputStreamReader(
                    Channels.newInputStream(
                    (new FileInputStream(FileDescriptor.in)).getChannel())));

            while (running) {
                String fromUser = null;
                try {
                    fromUser = in.readLine();
                } catch (IOException ex) {
                }
                if (fromUser != null && running) {
                    System.out.println("Client: " + fromUser);
                    o.println(fromUser);
                }
            }
            try {
                in.close();
            } catch (IOException ex) {
            }
        }
    }

    protected class HiveServerListener implements ServerListener {

        /**
         * {@inheritDoc}
         */
        @Override
        public synchronized void awaitLogin() {
            try {
                command = new ServerCommand(handler);
            } catch (IOException ex) {
                System.err.println("ERROR. Connection broken.");
                command = null;
                return;
            }
            try {
                // debug purposes only
                InputThread in = new InputThread(new PrintWriter(handler.getOut(), true));
                in.start();
            } catch (IOException ex) {
                System.err.println("ERROR. InputThread failed.");
            }
            command.login(name, password);
        }

        /**
         * {@inheritDoc}
         */
        @Override
        public void chatInfo(String message) {
        }

        /**
         * {@inheritDoc}
         */
        @Override
        public void chatJoinChannel(String channel, String who) {
        }

        /**
         * {@inheritDoc}
         */
        @Override
        public void chatLeaveChannel(String channel, String name) {
        }

        /**
         * {@inheritDoc}
         */
        @Override
        public void chatMessage(String channel, String from, String message) {
        }

        @Override
        public synchronized void logout(String string) {
            if (string == null) {
                command = null;
                clientHiveModel = null;
                hiveModel = null;
                gameList.clear();
                //Buttons.StartOfflineGame.setEnabled(true);
                Buttons.StartOnlineGame.setEnabled(false);
                //Buttons.LoadReplay.setEnabled(true);
                handler = null;
                name = "";
                password = "";
                statusBar.setStatusText(0, "Verbindung zum Server getrennt.");
                Buttons.Connect.setSelected(false);
            }
        }

        /**
         * {@inheritDoc}
         */
        @Override
        public synchronized void loginName(String name) {
            JHiveFrame.this.name = name;
        }

        /**
         * {@inheritDoc}
         */
        @Override
        public synchronized void gameCreated(String name) {
            gameList.addElement(name);
        }

        /**
         * {@inheritDoc}
         */
        @Override
        public synchronized void games(String[] parts) {
            gameList.clear();
            for (int i = 1; i < parts.length; ++i) {
                String s = parts[i];
                int pos = s.indexOf(',');
                gameList.addElement(s.subSequence(0, pos));
            }
        }

        /**
         * {@inheritDoc} Note that failed and success messages are personalized
         * and only sent to players who did an action. Otherwise, events are
         * sent to all players!
         */
        @Override
        public synchronized void success(Success success) {
            switch (success) {
                case CreateGame:
                    if (startGameDialog == null) {
                        return;
                    }
                    startGameDialog.createSuccess(true);
                    break;
                case JoinGame:
                    if (startGameDialog == null) {
                        return;
                    }
                    startGameDialog.joinSuccess(true);
                    break;
                case Login:
                    //Buttons.StartOfflineGame.setEnabled(false);
                    Buttons.StartOnlineGame.setEnabled(true);
                    //Buttons.LoadReplay.setEnabled(false);
                    statusBar.setStatusText(0, "Login als " + name + " auf " + handler.getServer() + " erfolgreich.");
                    break;
                default:
                    throw new IllegalArgumentException("Unsupported success message");
            }
        }

        /**
         * {@inheritDoc} Note that failed and success messages are personalized
         * and only sent to players who did an action. Otherwise, events are
         * sent to all players!
         */
        @Override
        public synchronized void failed(Failed failed) {
            switch (failed) {
                case Connect:
                    statusBar.setStatusText(0, "Verbindung zu " + handler.getServer() + " nicht möglich.");
                    handler = null;
                    Buttons.Connect.setSelected(false);
                    break;
                case Create:
                    if (startGameDialog == null) {
                        return;
                    }
                    startGameDialog.createSuccess(false);
                    statusBar.setStatusText(0, "Spiel erstellen fehlgeschlagen.");
                    break;
                case Join:
                    if (startGameDialog == null) {
                        return;
                    }
                    startGameDialog.joinSuccess(false);
                    statusBar.setStatusText(0, "Beitreten fehlgeschlagen.");
                    break;
            }
        }

        /**
         * {@inheritDoc}
         */
        @Override
        public synchronized void joinGame(String gameName, String playerName) {
            if (startGameDialog != null) {
                startGameDialog.joinGame(gameName, playerName);
            }
        }

        /**
         * {@inheritDoc}
         */
        @Override
        public synchronized void startGame(String game, String playerName) {
            if (startGameDialog == null || !game.equals(startGameDialog.getName())) {
                return;
            }
            JOptionPane.showMessageDialog(JHiveFrame.this, "Das Spiel beginnt!");
            replay = null;
            upper.reset();
            startGameDialog.startGame(game);
            // here the dialog is hidden, but this is not interesting for us here!
            // this method should been called out of the action handler as a result
            // to the EVENT GAME_STARTED
            // so, here we can set up the client hive model, all later messages
            // will be queued up, until this is done. after this, we return
            // to the message handler and deal with other messages
            clientHiveModel = GameFactory.createNewClientHiveModel(startGameDialog.getName(), command);
            hiveModel = clientHiveModel;
            hiveModel.addObserver(JHiveFrame.this);
            upper.setHiveModel(clientHiveModel);
            if (startGameDialog.isStartPlayer()) {
                upper.startNewGame(PlayerColor.WHITE, true);
            } else {
                upper.startNewGame(PlayerColor.BLACK, true);
            }
            upper.setEnabled(true);
        }

        /**
         * {@inheritDoc}
         */
        @Override
        public synchronized void hiveWhiteFirst(String game, String player, PieceType pieceType) {
            if (JHiveFrame.this.clientHiveModel == null || JHiveFrame.this.hiveModel == null)//hier reingeschrieben, weil die GUI manchmal zu langsam agiert und nsi bei dieser Nachricht des Server noch nicht initialisiert ist
            {
                while (JHiveFrame.this.clientHiveModel == null || JHiveFrame.this.hiveModel == null) {
                    try {
                        Thread.sleep(50);
                    } catch (InterruptedException e) {
                        System.err.println("HiveWindow caught InterruptedException in method hiveWhiteFirst" + e.getMessage());
                    }
                }
            }

            if (clientHiveModel != null && JHiveFrame.this.clientHiveModel.getGameName().equals(game)) {
                if (hiveModel.getCurrentPlayer() == PlayerColor.BLACK) {
                    throw new IllegalStateException("Illegal message order. Should receive black.");
                }
                clientHiveModel.introduceLocal(pieceType);
            }
        }

        /**
         * {@inheritDoc}
         */
        @Override
        public synchronized void hiveBlackFirst(String game, String player, PieceType pieceType, HexDirection dir) {
            if (clientHiveModel != null && clientHiveModel.getGameName().equals(game)) {
                if (hiveModel.getCurrentPlayer() == PlayerColor.WHITE) {
                    throw new IllegalStateException("Illegal message order. Should receive white first.");
                }
                clientHiveModel.introduceLocal(pieceType, 0, dir);
            }
        }

        /**
         * {@inheritDoc}
         */
        @Override
        public synchronized void hiveIntroducePiece(String game, String player, PieceType pieceType, int anchorPiece, HexDirection hexDirection) {
            if (clientHiveModel != null && clientHiveModel.getGameName().equals(game)) {
                clientHiveModel.introduceLocal(pieceType, anchorPiece, hexDirection);
            }
        }

        /**
         * {@inheritDoc}
         */
        @Override
        public synchronized void hiveMovePiece(String game, String player, int piece, int anchorPiece, HexDirection hexDirection) {
            if (clientHiveModel != null && clientHiveModel.getGameName().equals(game)) {
                clientHiveModel.moveLocal(piece, anchorPiece, hexDirection);
            }
        }

        @Override
        public synchronized void gameOver(String gameName, int result) {
            if (clientHiveModel != null && gameName.equals(clientHiveModel.getGameName())) {
                if (result == 0) {
                    JOptionPane.showMessageDialog(JHiveFrame.this, "Spieler weiss hat das Spiel gewonnen.", "Sieg!", JOptionPane.INFORMATION_MESSAGE);
                } else if (result == 1) {
                    JOptionPane.showMessageDialog(JHiveFrame.this, "Spieler schwarz hat das Spiel gewonnen.", "Sieg!", JOptionPane.INFORMATION_MESSAGE);
                } else {
                    JOptionPane.showMessageDialog(JHiveFrame.this, "Kein Spieler hat gewonnen.", "Unentschieden!", JOptionPane.INFORMATION_MESSAGE);
                }
            }
        }
    }

    protected class ToolBarActionListener implements ActionListener {

        @Override
        public void actionPerformed(ActionEvent e) {
            if (e.getActionCommand().equals("offline")) {
                clientHiveModel = null;
                replay = null;
                upper.reset();
                createNewHiveModel();
                upper.setHiveModel(hiveModel);
                upper.startNewGame(PlayerColor.WHITE, false);
                upper.setEnabled(true);
                hiveModel.setUndoAllowed(false);
            } else if (e.getActionCommand().equals("connect")) {
                if (!Buttons.Connect.isSelected()) {
                    JConnectDialog dialog = new JConnectDialog(JHiveFrame.this, "Mit Hive-Server verbinden");
                    dialog.setVisible(true);
                    if (dialog.getDialogResult() == JOptionPane.OK_OPTION) {
                        connect(dialog.getServer(), dialog.getPort(), dialog.getName(), dialog.getPassword());
                    }
                } else {
                    command.logout();
                }
            } else if (e.getActionCommand().equals("online")) {
                startGameDialog = new JStartGameDialog(JHiveFrame.this, "Hive-Spiel erstellen", gameList, command);
                startGameDialog.setVisible(true);
            } else if (e.getActionCommand().equals("zoomOut")) {
                zoom = Math.max(zoom - 1, -10);
                upper.zoom(1 + zoom * 0.1);
            } else if (e.getActionCommand().equals("zoomIn")) {
                zoom++;
                upper.zoom(1 + zoom * 0.1);
            } else if (e.getActionCommand().equals("hex_grid")) {
                upper.setDrawHexBorder(!upper.getDrawHexBorder());
            } else if (e.getActionCommand().equals("hex_numbers")) {
                upper.setDrawHexNumbers(!upper.getDrawHexNumbers());
            } else if (e.getActionCommand().equals("replay")) {
                if (replayFileChooser.showOpenDialog(JHiveFrame.this) == JFileChooser.APPROVE_OPTION) {
                    CompressedXMLFileReader<LinkedList<String>> a = new CompressedXMLFileReader<LinkedList<String>>(replayFileChooser.getSelectedFile());
                    upper.reset();
                    createNewHiveModel();
                    replay = new Replay(hiveModel);
                    replay.load(a);
                    upper.setHiveModel(hiveModel);
                    upper.startNewGame(PlayerColor.WHITE, false);
                    upper.setEnabled(false);
                    hiveModel.setUndoAllowed(true);
                    statusBar.setStatusText(0, "Replay mit " + replay.size() + " Schritten geladen.");
                }
            } else if (e.getActionCommand().equals("toEnd")) {
                while (true) {
                    if (!replay.replayOneStepForward()) {
                        break;
                    }
                }
            } else if (e.getActionCommand().equals("forwards")) {
                replay.replayOneStepForward();
            } else if (e.getActionCommand().equals("backwards")) {
                replay.replayOneStepBackward();
            } else if (e.getActionCommand().equals("toStart")) {
                while (true) {
                    if (!replay.replayOneStepBackward()) {
                        break;
                    }
                }
            } else if (e.getActionCommand().equals("exit")) {
                System.exit(0);
            }
            updateButtons();
        }
    }
}
