/**
 * JHivePanel.java
 *
 */
package client.gui;

import java.awt.BorderLayout;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import common.interfaces.HiveInterface;
import common.hive.PlayerColor;
import common.hive.PieceType;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 * A component that displays a {@link JBoardPanel} containing hexagonal elements for
 * and a {@link JPlayerPanel} to insert new piece.
 * @author Jan-Philipp Kappmeier
 */
@SuppressWarnings( "serial" )
public class JHivePanel extends JPanel {
	/** The board component. */
	private JBoardPanel board;
	/** The piece component. */
	private JPlayerPanel pieceHeap;

	/**
	 * Initializes the component with disabled components.
	 */
	public JHivePanel( ) {
		super( new BorderLayout() );
		initComponent();
	}

	/**
	 * Sets up the board within a scroll pane to allow zooming.
	 */
	private void initComponent() {
        int pieceCount = 4;
        for (PieceType type : PieceType.values()) {
            pieceCount += type.getNumberOfPieces() * 2;
        }        
		board = new JBoardPanel(pieceCount, pieceCount);
		JScrollPane backgroundScroll = new JScrollPane( board );
		add( backgroundScroll, BorderLayout.CENTER );

		pieceHeap = new JPlayerPanel( board );
        board.setPieceHeap(pieceHeap);
		add( pieceHeap, BorderLayout.SOUTH );
	}

	/**
	 * Sets the game model.
	 * @param hiveInterface the game model
	 */
	public void setHiveModel( HiveInterface hiveInterface ) {
		board.setHiveInterface( hiveInterface );
		pieceHeap.setHiveInterface( hiveInterface );
		hiveInterface.addObserver( board );
		hiveInterface.addObserver( pieceHeap );
	}

	/**
	 * Starts a new game. Enables the components and initializes them to the start status.
	 * @param playerColor the player color that is initially displayed
	 * @param networkGame decides if the new game is offline or online
	 * @throws IllegalArgumentException if playerColor is the black player and no network game is started
	 */
	public void startNewGame( PlayerColor playerColor, boolean networkGame ) throws IllegalArgumentException {
		if( networkGame == false && playerColor == PlayerColor.BLACK )
			throw new IllegalArgumentException( "In offline game, alsways the white player starts." );
		board.setNetworkGame( playerColor, networkGame );
		pieceHeap.startGame( playerColor, networkGame );
	}

	/**
	 * Enables and disables the drawing of the hexagonal shapes of possibly
	 * piece positions.
	 * @param value decides whether the border shapes are to be drawn, or not
	 */
	public void setDrawHexBorder( boolean value ) {
		board.setDrawEmptyTiles(value);
        repaint();
	}

	/**
	 * Decides whether the border shapes are drawn, or not
	 * @return {@code true} if the border shapes are to be drawn, {@code false} otherwise
	 */
	public boolean getDrawHexBorder() {
		return board.isDrawEmptyTiles();
	}

	/**
	 * Enables and disables the drawing of the hexagonal shapes of possibly
	 * piece positions.
	 * @param value decides whether the border shapes are to be drawn, or not
	 */
	public void setDrawHexNumbers( boolean value ) {
		board.setDrawTileIndices(value);
        repaint();
	}

	/**
	 * Decides whether the border shapes are drawn, or not
	 * @return {@code true} if the border shapes are to be drawn, {@code false} otherwise
	 */
	public boolean getDrawHexNumbers() {
		return board.isDrawTileIndices();
	}

	/**
	 * Zooms the game board to the original size times a given factor.
	 * @param factor the zoom factor
	 */
	public void zoom( double factor ) {
		board.zoom( factor );
	}

	public void reset() {
		board.reset();
	}
}
