/*
 * JHiveToolBar.java
 *
 */
package client.gui;

import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.Insets;
import java.awt.Paint;
import java.awt.Rectangle;
import java.awt.TexturePaint;
import java.awt.event.ActionListener;
import java.awt.image.BufferedImage;
import javax.swing.JToolBar;

/**
 * A tool bar that can load some {@link Buttons} in a given order.
 */
@SuppressWarnings(value = "serial")
class JHiveToolBar extends JToolBar {
    /** The order of buttons and separators in the tool bar. */
    private Buttons[] buttonList = {Buttons.Exit, null, Buttons.StartOfflineGame, null, Buttons.ZoomOut, Buttons.ZoomIn, Buttons.HexGrid, Buttons.HexNumbers, null, Buttons.LoadReplay, Buttons.ToStart, Buttons.Backward, Buttons.Forward, Buttons.ToEnd, Buttons.ComputeAIMove};
    /** The action listener that is given to the buttons. */
    private ActionListener acl = null;

    /**
     * Creates the tool bar and sets the name to "Hive".
     */
    protected JHiveToolBar() {
        super("Hive");
        setFloatable(false);
        setupImagePainter(Images.getHoneycomb());
    }

    /**
     * Adds the buttons to the tool bar.
     */
    public void initialize() {
        removeAll();
        for (Buttons b : buttonList) {
            if (b != null) {
                b.addToToolBar(this);
                if (acl != null) {
                    b.setActionListener(acl);
                }
            } else {
                addSeparator();
            }
        }
    }

    /**
     * <p>Sets a new button order for the tool bar. The buttons are stored in an
     * array indicating their order. A {@code null} entry represents a
     * separator.</p>
     * <p>Note that the reference to the array is stored, it is not copied.</p>
     * @param buttonList the list of buttons.
     */
    @SuppressWarnings(value = "AssignmentToCollectionOrArrayFieldFromParameter")
    public void setButtons(Buttons[] buttonList) {
        this.buttonList = buttonList;
    }

    /**
     * Sets the {@link ActionListener} that is added to all of the buttons.
     * @param acl the action listener.
     */
    public void setAcl(ActionListener acl) {
        this.acl = acl;
    }

      private TexturePaint ourPainter = null;

  //public JTexturedPanel(LayoutManager layout, Image texture) {
    //super(layout);

  //}

  /**
   * Creates a new TexturePaint using the provided image.
   */
  private void setupImagePainter(Image texture) {
    if (texture == null) {
      ourPainter = null;
      return;
    }

    int w = texture.getWidth(this);
    int h = texture.getHeight(this);

    if (w <= 0 || h <= 0) {
      ourPainter = null;
      return;
    }

    BufferedImage buff = new BufferedImage(w, h, BufferedImage.TYPE_INT_ARGB_PRE);

    Graphics2D g2 = buff.createGraphics();
    g2.drawImage(texture, 0, 0, this);
    ourPainter = new TexturePaint(buff, new Rectangle(0, 0, w, h));

    g2.dispose();
  }

  /**
   * Paints this component with its textured background.
   */
  protected void paintComponent(Graphics g) {
    super.paintComponent(g);

    if (ourPainter != null) {
      int w = getWidth();
      int h = getHeight();
      Insets in = getInsets();

      int x = in.left;
      int y = in.top;
      w = w - in.left - in.right;
      h = h - in.top - in.bottom;

      if (w >= 0 && h >= 0) {
        Graphics2D g2 = (Graphics2D) g;
        Paint pt = g2.getPaint();
        g2.setPaint(ourPainter);
        g2.fillRect(x, y, w, h);
        g2.setPaint(pt);
      }
    }
  }

}
