/**
 * JPlayerPanel.java 
 */
package client.gui;

import info.clearthought.layout.TableLayout;
import java.awt.Dimension;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Observable;
import java.util.Observer;
import javax.swing.JButton;
import common.hive.PlayerColor;
import common.hive.PieceType;
import common.interfaces.HiveInterface;

/**
 * A simple component that allows selection of a piece that should be
 * introduced. A button for each piece is provided followed by a label
 * indicating the number of left pieces of the very type. Additionally the
 * current player is displayed and the buttons are enabled/disabled if
 * necessary.
 *
 * @author Jan-Philipp Kappmeier, Martin Gross
 */
@SuppressWarnings( "serial")
public class JPlayerPanel extends JTexturedPanel implements Observer {

    /**
     * The game board.
     */
    private JBoardPanel board;
    /**
     * The interface to the model (which receives the insertion order.
     */
    private HiveInterface si;
    /**
     * The color of the player playing with this instance in network mode.
     */
    private PlayerColor thisPlayer;
    /**
     * The current player.
     */
    private PlayerColor player = PlayerColor.WHITE;
    /**
     * Stores information if the first piece has already been introduced.
     */
    private boolean firstIntroduce = true;
    /**
     * A piece for the moves.
     */
    private int move = 1;
    /**
     * Decides if the current game is a network game, or not.
     */
    private boolean networkGame = false;
    /**
     * The space between the buttons in the GUI.
     */
    final private static int space = 10;
    /**
     * Positions of the buttons in the {@link TableLayout} layout manager.
     */
    private final static double[][] SIZE;
    static {
        SIZE = new double[2][PieceType.values().length * 2 + 2];
        for (int i = 0; i < SIZE[0].length; ++i) {
            if (i % 2 == 0) {
                SIZE[0][i] = space;
            } else {
                SIZE[0][i] = TableLayout.PREFERRED;
            }
        }
        SIZE[0][SIZE[0].length - 1] = TableLayout.FILL;
        SIZE[1] = new double[]{space, TableLayout.PREFERRED, space};
    }

    /**
     * Creates a new instance, with all elements are disabled.
     *
     * @param board the game board. used to highlight reachable fields.
     */
    JPlayerPanel(JBoardPanel board) {
        super(new TableLayout(SIZE), Images.getHoneycomb());
        this.board = board;
        initComponents();
    }
    private JButton[] pieceButtons;
    private Dimension[] preferredSizes;

    public void clearSelection() {
        for (PieceType type : PieceType.values()) {
            pieceButtons[type.ordinal()].setSelected(false);
        }
    }

    /**
     * Initializes the buttons and label texts.
     */
    private void initComponents() {
        pieceButtons = new JButton[PieceType.values().length];
        preferredSizes = new Dimension[PieceType.values().length];
        for (PieceType type : PieceType.values()) {
            pieceButtons[type.ordinal()] = new JButton(Images.getIconFor(type, PlayerColor.WHITE));
            pieceButtons[type.ordinal()].addActionListener(aclAddPiece);
            pieceButtons[type.ordinal()].setActionCommand(type.toString());
            pieceButtons[type.ordinal()].setBorderPainted(false);
            pieceButtons[type.ordinal()].setContentAreaFilled(false);
            pieceButtons[type.ordinal()].setEnabled(false);
            pieceButtons[type.ordinal()].setFocusPainted(false);
            pieceButtons[type.ordinal()].setIconTextGap(0);
            pieceButtons[type.ordinal()].setMargin(new Insets(2, 2, 2, 2));
            pieceButtons[type.ordinal()].setOpaque(false);
            add(pieceButtons[type.ordinal()], (2 * type.ordinal() + 1) + ",1");
        }
    }

    /**
     * Starts a new game.
     *
     * @param playerColor the color of the current player. It can be black, in
     * network mode.
     * @param networkGame decides if a network game starts
     * @throws IllegalArgumentException if in an offline game the black player
     * should start.
     */
    void startGame(PlayerColor playerColor, boolean networkGame) throws IllegalArgumentException {
        if (networkGame == false && playerColor == PlayerColor.BLACK) {
            throw new IllegalArgumentException("In offline game, alsways the white player starts.");
        }
        firstIntroduce = true;
        for (PieceType type : PieceType.values()) {
            preferredSizes[type.ordinal()] = pieceButtons[type.ordinal()].getPreferredSize();
        }
        player = playerColor;
        thisPlayer = playerColor;
        this.networkGame = networkGame;
        move = 0;
        updatePlayer();
    }

    /**
     * Toggles the current player. The status of the buttons and the label texts
     * are updated.
     */
    private void updatePlayer() {
        move++;
        player = si.getCurrentPlayer();
        for (PieceType type : PieceType.values()) {
            boolean queenNotForced = (move != 7 && move != 8) || si.getRemainingPieces(player, PieceType.QUEEN) == 0;
            boolean enabled = si.getRemainingPieces(player, type) > 0 && (queenNotForced || type == PieceType.QUEEN) && (move > 2 || type != PieceType.QUEEN) && (!networkGame || player == thisPlayer) && !si.isGameOver();
            pieceButtons[type.ordinal()].setEnabled(enabled);
            pieceButtons[type.ordinal()].setIcon(Images.getIconFor(type, player, si.getRemainingPieces(player, type), false));
            pieceButtons[type.ordinal()].setMinimumSize(preferredSizes[type.ordinal()]);
            pieceButtons[type.ordinal()].setPreferredSize(preferredSizes[type.ordinal()]);
            pieceButtons[type.ordinal()].setSelected(false);
            pieceButtons[type.ordinal()].setSelectedIcon(Images.getIconFor(type, player, si.getRemainingPieces(player, type), true));
            pieceButtons[type.ordinal()].setToolTipText("" + si.getRemainingPieces(player, type));
        }
    }

    /**
     * Called by the observed model if a move occurred and the button status
     * should be updated.
     *
     * @param o the observable object, not used
     * @param arg the argument, not used
     */
    public void update(Observable o, Object arg) {
        updatePlayer();
        if (firstIntroduce) // if we receive this without introducing ourselves, the first figure was put remote.
        {
            firstIntroduce = false;
        }
    }

    /**
     * Sets a new hive interface connecting to the model.
     *
     * @param si the hive interface
     */
    void setHiveInterface(HiveInterface si) {
        this.si = si;
    }
    /**
     * The action listener listening to button clicks. The insertion commands
     * are pushed further to the model and the board.
     */
    private ActionListener aclAddPiece = new ActionListener() {
        @Override
        public void actionPerformed(ActionEvent e) {
            final PieceType ct = PieceType.valueOf(e.getActionCommand());
            if (firstIntroduce) {
                board.introduce(ct);
                firstIntroduce = false;
            } else {
                for (PieceType type : PieceType.values()) {
                    pieceButtons[type.ordinal()].setSelected(e.getSource() == pieceButtons[type.ordinal()]);
                }
                board.highlightInsertionFields(ct);
            }
            board.repaint();
        }
    };
}
