/**
 * JStartGameDialog.java
 *
 */
package client.gui;

import client.network.ServerCommandInterface;
import info.clearthought.layout.TableLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Frame;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.border.BevelBorder;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

/**
 * A swing dialog that allows to start or join a network game. In a list, all
 * active games are listed. Either one of these games can be joined or a new
 * game can be created. The necessary commands are automatically sent to the
 * server.
 *
 * @author Jan-Philipp Kappmeier, Martin Gross
 */
@SuppressWarnings( "serial")
public class JStartGameDialog extends JDialog {

    /**
     * A list model for the game list.
     */
    private DefaultListModel gameList;
    /**
     * The object that allows communication with the server.
     */
    private ServerCommandInterface command;
    /**
     * An input field for a game name (if a game should be created).
     */
    private JTextField txtName;
    /**
     * A button for joining a selected game.
     */
    private JButton btnJoin;
    /**
     * A list box with the available games.
     */
    private JList gameListBox;
    /**
     * A button that creates a game.
     */
    private JButton btnCreate;
    /**
     * Starts a game, if one was successfully created before.
     */
    private JButton btnStart;
    /**
     * Gives out some status text.
     */
    private JLabel lblStatus;
    /**
     * Decides if a game was joined successfully.
     */
    private boolean gameJoined;
    /**
     * Decides if a game was created successfully.
     */
    private boolean gameCreated;
    /**
     * The name of the game that was created.
     */
    private String name;
    /**
     *
     */
    private boolean startPlayer;

    /**
     * Sets up the game start dialog. Besides the default window information
     * about the parent window and the title, a list model containing the
     * available matches and a server command object is submitted.
     *
     * @param owner the parent of the dialog
     * @param title the title of the dialog
     * @param list a list containing the games already started. the list is
     * updated from outside the dialog
     * @param command the object that allows sending commands to the server
     */
    public JStartGameDialog(Frame owner, String title, DefaultListModel list, ServerCommandInterface command) {
        super(owner, title, true);
        final int width = 530;
        final int height = 500;
        setSize(width, height);
        setLocation(owner.getX() + (owner.getWidth() - width) / 2, owner.getY() + (owner.getHeight() - height) / 2);
        this.command = command;
        this.gameList = list;
        initComponents();
    }

    /**
     * Initializes the swing components in the window.
     */
    private void initComponents() {
        int space = 12;

        double[][] size = {
            {space, TableLayout.PREFERRED, space, TableLayout.FILL, space},
            {space, TableLayout.PREFERRED, space, TableLayout.PREFERRED, space, TableLayout.PREFERRED, 4, TableLayout.FILL, space, TableLayout.PREFERRED, space, TableLayout.PREFERRED, space}
        };

        JTexturedPanel mainPanel = new JTexturedPanel(new TableLayout(size), Images.getHoneycomb());
        add(mainPanel);

        gameListBox = new JList(gameList);
        gameListBox.addListSelectionListener(new ListSelectionListener() {

            @Override
            public void valueChanged(ListSelectionEvent e) {
                btnJoin.setEnabled(!gameListBox.isSelectionEmpty() && !gameCreated && !gameJoined);
            }
        });
        btnCreate = new JButton("Erstellen");
        btnCreate.setActionCommand("create");
        btnCreate.addActionListener(acl);
        btnStart = new JButton("Starten");
        btnStart.setActionCommand("start");
        btnStart.addActionListener(acl);
        btnStart.setEnabled(false);

        btnJoin = new JButton("Beitreten");
        btnJoin.setActionCommand("join");
        btnJoin.addActionListener(acl);
        btnJoin.setEnabled(false);

        txtName = new JTextField("");
        int counter = 0;
        while (txtName.getText().isEmpty() || gameList.contains(txtName.getText())) {
            txtName.setText("Testspiel " + (++counter));
        }
        txtName.getDocument().addDocumentListener(new DocumentListener() {

            @Override
            public void insertUpdate(DocumentEvent e) {
                btnCreate.setEnabled(!gameList.contains(txtName.getText()) && !gameCreated && !gameJoined);
            }

            @Override
            public void removeUpdate(DocumentEvent e) {
                btnCreate.setEnabled(!gameList.contains(txtName.getText()) && !gameCreated && !gameJoined);
            }

            @Override
            public void changedUpdate(DocumentEvent e) {
                btnCreate.setEnabled(!gameList.contains(txtName.getText()) && !gameCreated && !gameJoined);
            }
        });
        JLabel lblCreate = new JLabel("Spiel erstellen:");
        lblCreate.setBackground(Color.LIGHT_GRAY.brighter());
        lblCreate.setBorder(new BevelBorder(BevelBorder.LOWERED) {
            @Override
            public Insets getBorderInsets(Component c, Insets insets) {
                Insets i = super.getBorderInsets(c, insets);
                i.set(i.top + 1, i.left + 3, i.bottom + 1, i.right + 2);
                return i;
            }
        });
        lblCreate.setOpaque(true);
        JLabel lblList = new JLabel("Verfügbare Spiele:");
        lblList.setBackground(Color.LIGHT_GRAY.brighter());
        lblList.setBorder(new BevelBorder(BevelBorder.LOWERED) {
            @Override
            public Insets getBorderInsets(Component c, Insets insets) {
                Insets i = super.getBorderInsets(c, insets);
                i.set(i.top + 1, i.left + 3, i.bottom + 1, i.right + 2);
                return i;
            }
        });
        lblList.setOpaque(true);
        lblStatus = new JLabel("0 Spieler");
        lblStatus.setBackground(Color.LIGHT_GRAY.brighter());
        lblStatus.setBorder(new BevelBorder(BevelBorder.LOWERED) {
            @Override
            public Insets getBorderInsets(Component c, Insets insets) {
                Insets i = super.getBorderInsets(c, insets);
                i.set(i.top + 1, i.left + 3, i.bottom + 1, i.right + 2);
                return i;
            }
        });
        lblStatus.setOpaque(true);
        mainPanel.add(lblCreate, "1,1");
        mainPanel.add(txtName, "3,1");
        mainPanel.add(btnCreate, "1,3");
        mainPanel.add(btnStart, "3,3");
        mainPanel.add(lblList, "1,5,3,5");
        JScrollPane pane = new JScrollPane(gameListBox);
        mainPanel.add(pane, "1,7,3,7");
        mainPanel.add(btnJoin, "1,9,3,9");
        mainPanel.add(lblStatus, "1,11,3,11");
    }
    
    /**
     * The action listener listening for pressed buttons on the dialog. These
     * events disables the elements on the dialog. If a success was received
     * from the server, these elements are enabled then.
     */
    private ActionListener acl = new ActionListener() {
        public void actionPerformed(ActionEvent e) {
            if (e.getActionCommand().equals("create")) {
                command.createGame(name = txtName.getText());
                btnJoin.setEnabled(false);
                txtName.setEnabled(false);
                btnCreate.setEnabled(false);
            } else if (e.getActionCommand().equals("start")) {
                command.startGame(txtName.getText());
            } else if (e.getActionCommand().equals("join")) {
                gameListBox.setEnabled(false);
                name = (String) gameListBox.getSelectedValue();
                command.joinGame(name);
                btnJoin.setEnabled(false);
                btnCreate.setEnabled(false);
                txtName.setEnabled(false);
            } else {
                throw new UnsupportedOperationException("Command " + e.getActionCommand() + " not supported yet.");
            }
        }
    };

    /**
     * Called if a join command was successful, or not. In both cases the
     * enabled elements on the dialog are set correctly
     *
     * @param value the status of the join
     */
    public void joinSuccess(boolean value) {
        if (value) {
            lblStatus.setText("Beigetreten.");
            gameJoined = true;
        } else {
            lblStatus.setText("Join fehlgeschlagen.");
            btnJoin.setEnabled(true);
            txtName.setEnabled(true);
            btnCreate.setEnabled(true);
            gameJoined = false;
            gameCreated = false;
        }
    }

    /**
     * Called if a game was created. Either if creation was successful or not,
     * the elements on the dialog are enabled correctly.
     *
     * @param value
     */
    public void createSuccess(boolean value) {
        if (value) {
            lblStatus.setText("Erstellt und beigetreten. Warte auf weiteren Spieler.");
            btnStart.setEnabled(false);
            gameCreated = true;
        } else {
            lblStatus.setText("Erstellen fehlgeschlagen.");
            btnJoin.setEnabled(true);
            txtName.setEnabled(true);
            btnCreate.setEnabled(true);
            gameListBox.setEnabled(true);
            gameJoined = false;
            gameCreated = false;
        }
    }

    /**
     * Called, if a player joined a game. If the player joined our game, we can
     * start. Thus this enables the start button.
     *
     * @param gameName the name of the game
     * @param playerName the player that joined a game
     */
    void joinGame(String gameName, String playerName) {
        if (gameCreated && gameName.equals(txtName.getText())) {
            // another playerh as joined our game
            btnStart.setEnabled(true);
            lblStatus.setText("Spieler beigetreten. Start möglich.");
        }
    }

    /**
     * Is called if a game was started. Some status variables are set correctly
     * and the dialog is closed (hidden).
     *
     * @param name the game that was started. If it does not equal the game that
     * we have created or joined, this call is ignored.
     */
    public void startGame(String name) {
        if (gameCreated && name.equals(txtName.getText())) {
            startPlayer = true;
            // created game started
            setVisible(false);
        } else if (gameJoined && ((String) gameListBox.getSelectedValue()).equals(name)) {
            // joined game started
            startPlayer = false;
            setVisible(false);
        } // otherwise, event was for another game
    }

    protected void updateButtons() {
    JButton btnJoin;

     JList gameListBox;

     JButton btnCreate;

     JButton btnStart;
    }

    /**
     * Decides if this player is the start player of the game, or not.
     *
     * @return {@code true}, if the current player starts the game,
     * {@code false} otherwise
     */
    public boolean isStartPlayer() {
        return startPlayer;
    }

    /**
     * The name of the game that is started.
     *
     * @return the name of the game that is started
     */
    @Override
    public String getName() {
        return name;
    }
}
