/**
 * ServerCommandInterface.java
 * Created: 16.05.2011, 11:12:23
 */

package client.network;

import common.hive.HexDirection;
import common.hive.PieceType;

/**
 * An interface providing methods that are sent to the server. Currently,
 * methods to login to the server, to allow basic chat features, start playing
 * a hive game and pass game moves are provided.
 * @author Jan-Philipp Kappmeier
 */
public interface ServerCommandInterface {

	/**
	 * <p>Use this method to log in to the server with a given name and password.</p>
	 * <p>When both, name and username are empty strings or null, the login is
	 * performed anonymously. The client receives a username of the form Guestxxx.</p>
	 * @param name the username
	 * @param password the password
	 */
	public void login( String name, String password );

	/**
	 * Logout from the server.
	 */
	public void logout();

	/**
	 * Send a message to a channel.
	 * @param channel the channel
	 * @param message the message text
	 */
	public void chat( String channel, String message );

	/**
	 * Join a specific channel (and automatically leave the old channel). Thus,
	 * this method helps providing a chat that only allows to be in one channel
	 * at a time.
	 * @param channel the new channel to join
	 * @param oldChannel the old channel
	 */
	public void chatJoinChannel( String channel, String oldChannel );

	/**
	 * Create a new (hive) game on the server with a specific name.
	 * @param name the name of the game
	 */
	public void createGame( String name );

	/**
	 * Join a previously created game of a given name
	 * @param name the name of the game
	 */
	public void joinGame( String name );

	/**
	 * Starts an already created game
	 * @param name the name of the game
	 */
	public void startGame( String name );

	/**
	 * Introduces the first piece in a game with a given name
	 * @param gameName the name of the game
	 * @param pieceType the piece type
	 */
	public void hiveIntroduce( String gameName, PieceType pieceType );

	/**
	 * Introduces the second piece (the first one for the black player) to a
	 * hive game with a specific name.
	 * @param gameName the name of the game
	 * @param pieceType the piece type
	 * @param dir the direction relative to the first piece
	 */
	public void hiveIntroduce( String gameName, PieceType pieceType, HexDirection dir );

	/**
	 * Introduces a piece to the game (except the first two). The piece are
	 * addressed relativeley to already placed pieces.
	 * @param gameName
	 * @param pieceType
	 * @param anchorPiece
	 * @param dir
	 */
	public void hiveIntroduce( String gameName, PieceType pieceType, int anchorPiece, HexDirection dir );

	/**
	 * Moves a piece in a specified hive game. The piece is specified by its index
	 * (order of introduction), the target field is addressed relativeley to an
	 * already introduced piece.
	 * @param gameName the name of the game
	 * @param piece the index of the piece that is to be moved
	 * @param anchorPiece the index of the piece that is used as an anchor for relative addressing
	 * @param dir the direction
	 */
	public void hiveMove( String gameName, int piece, int anchorPiece, HexDirection dir );
	
	/**
	 * Ask the server to send a list of all games currently created which can be joined. 
	 */
	public void getGames();
}
