/**
 * ServerListener.java
 * Created: 23.02.2011, 15:15:15
 */

package client.network;

import common.hive.PieceType;
import common.hive.HexDirection;

/**
 * An interface specifying all callback methods that a server listener should
 * implement.
 * @author Jan-Philipp Kappmeier, Martin Gross
 */
public interface ServerListener {
	/**
	 * Should be called when the server sends "AWAITING_LOGIN" when a connection
	 * is established.
	 */
	void awaitLogin();

	/**
	 * After a successful login, the server sends the name in the game, that must
	 * not be the same as the login name, together with the command "LOGGED_IN_AS".
	 * @param name the user name. this is also used in the chat.
	 */
	public void loginName( String name );

	/**
	 * Should be called as response to the "EVENT" command "CHANNEL_JOINED" of the server,
	 * that indicates that a channel is joined by a user.
	 * @param channel the channel that is joined
	 * @param who the user that joins the channel
	 */
	public void chatJoinChannel( String channel, String who );

	/**
	 * Should be called as response to the "EVENT" command "CHANNEL_LEFT" of the server,
	 * that indicates that a channel has been left by a user.
	 * @param channel the channel that has been left
	 * @param name the user that leaves the channel
	 */
	public void chatLeaveChannel( String channel, String name );

	/**
	 * The server sends a chat message using the "CHAT_MESSAGE" command. It contains
	 * the chat room where the massage was posted, the user who posted it and the
	 * message itself.
	 * @param channel the channel
	 * @param from the user posting the message
	 * @param message the message text
	 */
	public void chatMessage( String channel, String from, String message );

	/**
	 * When a new channel is joined, the server sends the "CHANNEL_INFO" command
	 * giving information about the channel in a specific format. This message
	 * can be parsed by a chat object.
	 * @param message the message sent by the server
	 */
	public void chatInfo( String message );

	/**
	 * States that a game has been created.
	 * @param name the name of the game
	 */
	public void gameCreated( String name );

	/**
	 * A list of all created games. the first entry is "GAMES" and does not
	 * mean a game.
	 * @param parts array of game information
	 */
	public void games( String[] parts );

	/**
	 * Called, if a player joined a game.
	 * @param gameName the name of the game
	 * @param playerName the player that joins the game
	 */
	public void joinGame( String gameName, String playerName );

	/**
	 * The "EVENT" that occurs if a game has been started is "GAME_STARTED"
	 * which gives the name of the newly started game.
	 * @param game the game name
	 * @param playerName the name of the player that has created the game
	 */
	public void startGame( String game, String playerName );

	/**
	 * In the "GAME" command in an "EVENT" for the game "HIVE", the
	 * "WHITE_FIRST_PIECE" command is sent if the first piece is set.
	 * @param game the game in which the piece is introduced
	 * @param player the player who introduces the piece
	 * @param pieceType the type of the first piece
	 */
	public void hiveWhiteFirst( String game, String player, PieceType pieceType );

	/**
	 * In the "GAME" command in an "EVENT" for the game "HIVE", the
	 * "BLACK_FIRST_PIECE" command is sent if the first black piece is set.
	 * @param game the game in which the piece is introduced
	 * @param player the player who introduces the piece
	 * @param pieceType the type of the piece
	 * @param dir the direction in which the new piece is placed, relative to the first piece
	 */
	public void hiveBlackFirst( String game, String player, PieceType pieceType, HexDirection dir );

	/**
	 * In the "GAME" command in an "EVENT" for the game "HIVE", the
	 * "INTRODUCE_PIECE" command is sent if a player introduces a piece (but
	 * not the first one, respectively).
	 * @param game the game in which a piece is introduced
	 * @param player the player who introduces the piece
	 * @param pieceType the type of the introduced piece
	 * @param anchorPiece a neighbor piece of the new piece
	 * @param hexDirection the direction in which the new piece is placed, relative to the given piece
	 */
	public void hiveIntroducePiece( String game, String player, PieceType pieceType, int anchorPiece, HexDirection hexDirection );

	/**
	 * In the "GAME" command in an "EVENT" for the game "HIVE", the
	 * "MOVE_PIECE" command is sent if a player moves a piece.
	 * @param game the game in which a piece is moved
	 * @param player the player who moves a piece
	 * @param piece the piece that is moved
	 * @param anchorPiece a neighbor piece at the target position
	 * @param hexDirection the direction in which the moved piece is placed, relative to the given piece
	 */
	public void hiveMovePiece( String game, String player, int piece, int anchorPiece, HexDirection hexDirection );

	/**
	 * A success message by the server. The parameter contains more detailed
	 * information of what was successful.
	 * @param success the success object
	 */
	public void success( Success success );

	/**
	 * An error message by the server. The parameter contains more detailed
	 * information of what went wrong.
	 * @param failed the failure object
	 */
	public void failed( Failed failed );

	/**
	 * Called, when a user is logged out from the server. This is an message handling
	 * an event!
	 * @param string the user name
	 */
	public void logout( String string );

	/**
	 * Called if a game is over.
	 * @param gameName the name of the game that is over.
	 * @param result 0 if white wins, 1 if black wins, 2 if the game ended with a draw
	 */
	public void gameOver( String gameName, int result );
}
