/**
 * Tuple.java
 * 
 */
package common;

/**
 * A two tuple storing two types of data together.
 *
 * @param <U> the data type of the first data entry.
 * @param <V> the data type of the second data entry.
 * @author Jan-Philipp Kappmeier, Martin Gross
 */
public class Tuple<U, V> {

    /**
     * The first data entry.
     */
    private U first;
    /**
     * The second data entry
     */
    private V second;

    /**
     * *
     * Creates the instance and takes the two data entries.
     *
     * @param u the first data entry
     * @param v the second data entry
     */
    public Tuple(U u, V v) {
        this.first = u;
        this.second = v;
    }

    /**
     * Returns the first data entry.
     *
     * @return the first data entry
     */
    public U getFirst() {
        return first;
    }

    /**
     * Returns the second data entry.
     *
     * @return the second data entry
     */
    public V getSecond() {
        return second;
    }

    /**
     * Compares this tuple to an other object for equality.
     * @param obj the object to check equality with.
     * @return <code>true<code> if the specified object is a tuple and it is 
     * component-wise equal (in regard to <code>equals</code>) to this tuple,
     * <code>false</code> otherwise. 
     */
    @Override
    public boolean equals(Object obj) {
        if (obj == null) {
            return false;
        }
        if (getClass() != obj.getClass()) {
            return false;
        }
        final Tuple<U, V> other = (Tuple<U, V>) obj;
        if (this.getFirst() != other.getFirst() && (this.getFirst() == null || !this.first.equals(other.first))) {
            return false;
        }
        if (this.getSecond() != other.getSecond() && (this.getSecond() == null || !this.second.equals(other.second))) {
            return false;
        }
        return true;
    }

    /**
     * Generates a hash code for this tuple based on the hash codes of its 
     * components.
     * @return a hash code for this tuple.
     */
    @Override
    public int hashCode() {
        int hash = 7;
        hash = 59 * hash + ((first != null)? first.hashCode() : 0);
        hash += 13 * ((second != null)? second.hashCode() : 0);
        return hash;
    }

    /**
     * Returns a string representation of the tuple in the form "(u,v)".
     *
     * @return a string representation of the tuple.
     */
    @Override
    public String toString() {
        return "(" + getFirst() + ',' + getSecond() + ')';
    }
}
