/**
 * CenteredHexField.java
 * 
 */
package common.hexfield;

import common.interfaces.DataContainer;
import common.Tuple;
import java.util.ArrayList;
import java.util.Arrays;

/**
 * <p>A specialized hex field that has the ability to center the occupied fields.
 * When one of the fields near the border is used (the data is not {@code null}),
 * it can translate all fields to center the used fields.</p>
 * <p>The data must be stored in a special class {@link common.DataContainer}</p>.
 * @param <T> The data that is stored in the hex field
 * @param <U> the corresponding {@link DataContainer}
 * @author Jan-Philipp Kappmeier
 */
public class CenteredHexField<T,U extends DataContainer<T>> extends HexField<U> {

	/**
	 * Creates a new hex field with the given dimensions. For generic array creation,
	 * the class of the type has to be submitted.
	 * @param rows the number of rows
	 * @param columns the number of columns
	 * @param type the type of the data container
	 */
	public CenteredHexField( int rows, int columns, Class<U> type ) {
		super( rows, columns, type );
	}

	/**
	 * Checks if a specified hex field on the two border lines are used. The returned value
	 * is a number that indicates the number of fields, that every hex field has
	 * to be removed to center the data. The number is in terms of the indices.
	 * @param position an index that is checked
	 * @return the number of indices that the used fields have been moved
	 */
	public int checkDangerousPosition( int position ) {
		// alert
		int x = getX( position );
		int y = getY( position );
		//System.out.println( "Placed at coordinate (" + x + "," + y + ")" );
		if( x == 0 || x == 1 || x == getWidth()-1 || x == getWidth()-2 || y == 0 || y == 1 || y == getHeight()-1 || y == getHeight()-2 ) {
			System.out.println( "Position=" + position );
			x = getX( position );
			y = getY( position );
			return recenter();
		} else
			return 0;
	}

	/**
	 * Recenters the used fields.
	 * @return the number of indices that the data in the fields has been moved.
	 */
	private int recenter() {
		// may easyly optimized
		int minx = Integer.MAX_VALUE;
		int maxx = -1;
		int miny = Integer.MAX_VALUE;
		int maxy = -1;
		for( DataContainer<?> t : this ) {
			if( t.containsData() ) {
				int x = getX( t.getIndex() );
				int y = getY( t.getIndex() );
				if( x < minx )
					minx = x;
				if( x > maxx )
					maxx = x;
				if( y < miny )
					miny = y;
				if( y > maxy )
					maxy = y;
			}
		}
		// compute offsets
		int xAbs = maxx - minx + 1;
		int yAbs = maxy - miny + 1;

		int h = getHeight();
		int possible = h - yAbs;
		int targetPos = possible/2;
		int movey = targetPos - miny;

		System.out.println( "Moving up by " + movey );

		int w = getWidth();
		possible = w - xAbs;
		targetPos = possible/2;
		int movex = targetPos - minx;

		if( movex % 2 != 0 )
			movex += movex > 0 ? 1 : -1;

		System.out.println( "Moving left by " + movex );

		int move = movey * w + movex;
		translate( move );
		return move;
	}

	/**
	 * Translates the data of all fields by a specific amount of indices.
	 * @param x how far the data is moved.
	 */
	protected void translate( int x ) {
		ArrayList<Tuple<Integer,T>> a = new ArrayList<Tuple<Integer,T>>();
		for( int i = 0; i < size(); ++i ) {
			if( get( i ).containsData() ) {
				a.add( new Tuple<Integer,T>( i+x, get( i ).getData() ) );
				get( i ).resetData();
			}
		}
		for( Tuple<Integer,T> t : a )
			get( t.getFirst()).setNewData( t.getSecond());
	}

	/**
	 * Translates all fields in an array by the specific amount of indices.
	 * @param fields the array of the fields that contain the data that is moved
	 * @param x the amount of indices that the data is moved
	 */
	protected void translate( int[] fields, int x ) {
		System.out.println( "Translating by " + x );
		System.out.println( "Fields: " + Arrays.toString( fields ) );
		
		if( x < 0 ) {
			for( int i : fields ) {
				System.out.print( i + "->" + (i + x) + ";" );
				get( i ).moveDataTo( get( i + x) );
			}
			System.out.println();
		} else {
			for( int i = fields.length-1; i >= 0 ; --i ) {
				System.out.print( fields[i] + "->" + (fields[i] + x) + ";" );
				get( fields[i] ).moveDataTo( get( fields[i] + x ) );
			}
			System.out.println();
		}
	}
}
