/**
 * HexField.java
 * Created: 10.02.2011, 16:21:53
 */
package common.hexfield;

import common.interfaces.HexFieldInterface;
import java.lang.reflect.Array;
import java.util.Iterator;
import common.hive.HexDirection;

/**
 * A hexagonal array stored in a quadratic array. Neighbor fields can be
 * accessed using the {@link model.ds.HexDirection}. Additionally access is
 * possible via row and column, directly by a number. Note that every second
 * row is shifted by a half due to the hexagonal organization.
 * @param <T> the type of data stored in the array
 * @author Jan-Philipp Kappmeier
 */
public abstract class HexField<T> implements HexFieldInterface<T> {
	/** The number of columns in the hex field. */
	private int width;
	/** The number of rows in the hex field. */
	private int height;
	/** The array containing the data. */
	private T[][] hexArray;

	/**
	 * Initializes a hexagonal field containing elements of a given type.
	 * @param rows the number of rows in the hexagonal array
	 * @param columns the number of columns in the hexagonal array
	 * @param type the type of the array elements. Must coincide to the generic parameter, otherwise {@link #set(int, int, java.lang.Object)} would fail
	 */
	public HexField( int rows, int columns, Class<T> type ) {
		hexArray = createArray( type, rows, columns );
		this.width = columns;
		this.height = rows;
	}

	/**
	 * Initializes the array with the generic type.
	 * @param classType the type of the data in the array
	 * @param rows the height of the array
	 * @param columns the width of the array
	 * @return the created array
	 */
	@SuppressWarnings({"unchecked"})
	private T[][] createArray(Class<T> classType, int rows, int columns) {
		return (T[][]) Array.newInstance(classType, new int[] {rows,columns} );
	}

	/**
	 * Sets the data object for a specified position.
	 * @param i the row
	 * @param j the column
	 * @param data the data stored in the field
	 */
    @Override
	public void set( int i, int j, T data ) {
		hexArray[i][j] = data;
	}

    @Override
	public void set( int i, T data ) {
		hexArray[i/width][i%width] = data;
	}

	/**
	 * Returns the data object for a specified position.
	 * @param i the row
	 * @param j the column
	 * @return the data object stored in the field
	 */
    @Override
	public T get( int i, int j ) {
		return hexArray[i][j];
	}

	/**
	 * Returns the data object for a specified field. The fields are counted from
	 * left to right and top to bottom.
	 * @param i the index of the field
	 * @return the data object stored in the field
	 */
    @Override
	public T get( int i ) {
		return hexArray[i/width][i%width];
	}

	/**
	 * Returns the number of hex fields in the hex array.
	 * @return the number of hex fields in the hex array
	 */
	public int size() {
		return width*height;
	}

	/**
	 * Returns the field in the center of the field. If the number of rows or
	 * columns is even, it is rounded down.
	 * @return the hex field in the center
	 */
	public int getCenter() {
		return (height/2)*width + width/2;
	}
	
	public T getNeighbor( int index, HexDirection dir ) {
		return get( getNeighborIndex( index, dir ) );
	}

	/**
	 * Returns a hex field adjacent to the hex field with a given index. The
	 * neighbor is specified as a direction.
	 * @param index the index of the starting field
	 * @param dir the direction
	 * @return the adjacent hex field in the given direction
	 */
    @Override
	public int getNeighborIndex( int index, HexDirection dir ) {
		int col = index % width;
		int row = index / width;
		switch( dir ) {
			case UPPER:
				return index - width;
			case LOWER:
				return index + width;
			case UPPER_RIGHT:
				if( col % 2 == 0 )
					return index - width+1;
				else
					return index+1;
			case UPPER_LEFT:
				if( col % 2 == 0 )
					return index - width-1;
				else
					return index-1;
			case LOWER_RIGHT:
				if( col % 2 == 0)
					return index + 1;
				else
					return index + width + 1;
			case LOWER_LEFT:
				if( col % 2 == 0)
					return index - 1;
				else
					return index + width - 1;
		}
				throw new IllegalStateException( "Cannot occur" );
	}

	/**
	 * Returns the field which follows to the field {@code start} if we travel
	 * around {@code center} in clockwise direction.
	 * @param center the center of a hex-ring
	 * @param start the hex field whose neighbor we are searching
	 * @return the next field in clockwise manner
	 * @throws IllegalArgumentException if center and start are not next to each other
	 */
	public int getClockWise( int center, int start ) {
		for( HexDirection d : HexDirection.values() )
			if( getNeighborIndex( center, d ) == start )
				return getNeighborIndex( center, HexDirection.values()[(d.ordinal()+1)%HexDirection.values().length] );
		throw new IllegalArgumentException( "Fields at the given position are not connected." );
	}

	/**
	 * Returns the field which follows to the field {@code start} if we travel
	 * around {@code center} in counter clockwise direction.
	 * @param center the center of a hex-ring
	 * @param start the hex field whose neighbor we are searching
	 * @return the next field in counter clockwise manner
	 * @throws IllegalArgumentException if center and start are not next to each other
	 */
	public int getCounterClockWise( int center, int start ) throws IllegalArgumentException {
		for( HexDirection d : HexDirection.values() )
			if( getNeighborIndex( center, d ) == start )
				return getNeighborIndex( center, HexDirection.values()[(d.ordinal()-1+HexDirection.values().length)%HexDirection.values().length] );
		throw new IllegalArgumentException( "Fields at the given position are not connected." );
	}

	/**
	 * Returns the direction in which the field with index end lies with respect
	 * to start. Returns {@code null} if they are not next to each other.
	 * @param start
	 * @param end
	 * @return
	 */
	public HexDirection getDirection( int start, int end ) {
		for( HexDirection d: HexDirection.values() ) {
			if( getNeighborIndex( start, d) == end )
				return d;
		}
		return null;
	}

	/**
	 * Returns an iterator that iterates over all hex fields.
	 * @return an iterator that iterates over all hex fields
	 */
	public Iterator<T> iterator() {
		return new DoubleArrayIterator<T>( hexArray );
	}

	/**
	 * Returns the height of the hex field which is defined as the number of rows.
	 * @return the number of columns
	 */
    @Override
	public int getHeight() {
		return height;
	}

	/**
	 * Returns the width of the hex field which is defined as the number of columns.
	 * @return
	 */
    @Override
	public int getWidth() {
		return width;
	}

	/**
	 * Returns the column for a field with a given index (starting from zero).
	 * @param pos the index of the field
	 * @return the column for the field with the given index
	 * @see #size()
	 */
	public int getX( int pos ) {
		return pos % width;
	}

	/**
	 * Returns the row for a field with a given index (starting from zero).
	 * @param pos the index of the field
	 * @return the row for the field with the given index
	 * @see #size()
	 */
	public int getY( int pos ) {
		return pos / width;
	}


    public abstract int checkDangerousPosition(int position);
}