/**
 * HexDirection.java
 * 
 */
package common.hive;

/**
 * <p>An enumeration for the six possible neighbors of a hexagonal field. The
 * elements have convenient methods to find the next, previous and opposite
 * direction.</p>
 * <p>The hex fields are ordered clockwise, beginning with the topmost.</p>
 *
 * @author Jan-Philipp Kappmeier, Martin Gross
 */
public enum HexDirection {

    /**
     * The (unique) field on above an hex field.
     */
    UPPER, /*0*/
    /**
     * The upper of the two fields on the right side.
     */
    UPPER_RIGHT, /*1*/
    /**
     * The lower of the two fields on the right side.
     */
    LOWER_RIGHT, /*2*/
    /**
     * The (unique) field below an hex field.
     */
    LOWER, /*3*/
    /**
     * The lower of the two fields on the left side.
     */
    LOWER_LEFT, /*4*/
    /**
     * The upper of the two fields on the left side.
     */
    UPPER_LEFT; /*5*/

    /**
     * Returns the opposite direction.
     *
     * @return the opposite direction
     * @throws IllegalStateException if any direction is not implemented
     */
    public HexDirection opposite() throws IllegalStateException {
        switch (this) {
            case UPPER:
                return LOWER;
            case UPPER_RIGHT:
                return LOWER_LEFT;
            case LOWER_RIGHT:
                return UPPER_LEFT;
            case LOWER:
                return UPPER;
            case LOWER_LEFT:
                return UPPER_RIGHT;
            case UPPER_LEFT:
                return LOWER_RIGHT;
        }
        throw new IllegalStateException("Direction does not exist.");
    }

    /**
     * Returns the next direction (which is right of the instance or in
     * clockwise order).
     *
     * @return the next direction.
     * @throws IllegalStateException if any direction is not implemented.
     */
    public final HexDirection next() throws IllegalStateException {
        switch (this) {
            case UPPER:
                return UPPER_RIGHT;
            case UPPER_RIGHT:
                return LOWER_RIGHT;
            case LOWER_RIGHT:
                return LOWER;
            case LOWER:
                return LOWER_LEFT;
            case LOWER_LEFT:
                return UPPER_LEFT;
            case UPPER_LEFT:
                return UPPER;
        }
        throw new IllegalStateException("Direction does not exist.");
    }

    /**
     * Returns the previous direction (left of the instances direction or in 
     * counter clockwise order).
     *
     * @return the previous direction.
     * @throws IllegalStateException if any direction is not implemented.
     */
    public final HexDirection previous() throws IllegalStateException {
        switch (this) {
            case UPPER:
                return UPPER_LEFT;
            case UPPER_RIGHT:
                return UPPER;
            case LOWER_RIGHT:
                return UPPER_RIGHT;
            case LOWER:
                return LOWER_RIGHT;
            case LOWER_LEFT:
                return LOWER;
            case UPPER_LEFT:
                return LOWER_LEFT;
        }
        throw new IllegalStateException("Direction does not exist.");
    }
}
