/**
 * PieceType.java
 * 
 */
package common.hive;

/**
 * An enumeration of the piece types available in Hive. Each piece stores the 
 * number of times that each player has access to it, and whether it can end its
 * turn on top of other pieces.
 *
 * @author Jan-Philipp Kappmeier, Martin Gross
 */
public enum PieceType {

    /**
     * The queen is unique and non capableOfRemainingOnOtherPieces.
     */
    QUEEN(1, false),
    /**
     * The bug exists twice and can be stacked onto other pieces.
     */
    BUG(2, true),
    /**
     * The grasshopper exists three times, it can jump over other pieces but is
     * non capableOfRemainingOnOtherPieces.
     */
    GRASSHOPPER(3, false),
    /**
     * The spider exists twice and can (limitedly) crawl around the used game
     * area.
     */
    SPIDER(2, false),
    /**
     * The ant exists three times and can run around the used game area.
     */
    ANT(3, false),
    /**
     * The ladybug exists once and moves three fields, twice on top of other
     * pieces and then onto a free field.
     */
    LADYBUG(1, false),
    /**
     * The mosquito exists once and copies movement methods from adjacent bugs.
     */
    MOSQUITO(1, true);

    /**
     * The number of pieces of this type.
     */
    private final int numberOfPieces;
    /**
     * Decides if this piece type can be placed on top of others.
     */
    private final boolean capableOfRemainingOnOtherPieces;

    /**
     * Sets the parameter for a piece type.
     *
     * @param count the number of pieces of this type.
     * @param capableOfRemainingOnOtherPieces if the piece can end its turn on 
     * top of other pieces.
     */
    private PieceType(int count, boolean capableOfRemainingOnOtherPieces) {
        this.numberOfPieces = count;
        this.capableOfRemainingOnOtherPieces = capableOfRemainingOnOtherPieces;
    }

    /**
     * Returns the name of the type in lower case letters.
     * @return the name of the type in lower case letters.
     */
    public String getLowerCaseName() {
        return toString().toLowerCase();
    }

    /**
     * @return the numberOfPieces
     */
    public int getNumberOfPieces() {
        return numberOfPieces;
    }

    /**
     * @return the capableOfRemainingOnOtherPieces
     */
    public boolean isCapableOfRemainingOnOtherPieces() {
        return capableOfRemainingOnOtherPieces;
    }
}
