/*
 * HexFieldInterface.java
 * 
 */
package common.interfaces;

import common.hive.HexDirection;

/**
 * An interface for a data structure representing a 2-dimensional field of 
 * hexagons, that can be iterated over.
 * 
 * @author Martin Gross
 * @param <T> the type of the hexagons.
 */
public interface HexFieldInterface<T> extends Iterable<T> {

	/**
	 * Checks if a specified hex field on the two border lines are used. The returned value
	 * is a number that indicates the number of fields, that every hex field has
	 * to be removed to center the data. The number is in terms of the indices.
	 * @param position an index that is checked
	 * @return the number of indices that the used fields have been moved
	 */
	int checkDangerousPosition( int position );    
    
    /**
     * Returns the data object for a specified position.
     * @param i the row
     * @param j the column
     * @return the data object stored in the field
     */
    T get(int i, int j);

    /**
     * Returns the data object for a specified field. The fields are counted from
     * left to right and top to bottom.
     * @param i the index of the field
     * @return the data object stored in the field
     */
    T get(int i);

 	/**
	 * Returns the field in the center of the field. If the number of rows or
	 * columns is even, it is rounded down.
	 * @return the hex field in the center
	 */   
    int getCenter();
    
    /**
     * Returns the height of the hex field which is defined as the number of rows.
     * @return the number of columns
     */
    int getHeight();

    /**
     * Returns a hex field adjacent to the hex field with a given index. The
     * neighbor is specified as a direction.
     * @param index the index of the starting field
     * @param dir the direction
     * @return the adjacent hex field in the given direction
     */
    int getNeighborIndex(int index, HexDirection dir);

    /**
     * Returns the width of the hex field which is defined as the number of columns.
     * @return
     */
    int getWidth();

    /**
     * Sets the data object for a specified position.
     * @param i the row
     * @param j the column
     * @param data the data stored in the field
     */
    void set(int i, int j, T data);

    void set(int i, T data);
    
}
