/**
 * HiveInterface.java
 *
 */
package common.interfaces;

import common.Tuple;
import common.hive.HexDirection;
import common.hive.PieceType;
import common.hive.PlayerColor;
import java.util.ArrayList;
import java.util.Observer;

/**
 * An interface for classes modeling hive games. The methods required by this 
 * interface are necessary for the GUI to work with it.
 *
 * @author Jan-Philipp Kappmeier, Martin Gross
 */
public interface HiveInterface {

    /**
     * Returns an anchor for the piece Q with the given index. An anchor is a 
     * tuple that consists of the index of another piece P, which is adjacent 
     * to the specified piece, and the direction D in which Q lies from P.
     * 
     * @param pieceIndex the piece for which the anchor is to be returned.
     * @return the anchor <P,D> for the piece. 
     */
    Tuple<Integer, HexDirection> getAnchor(int pieceIndex);

    /**
     * Adds an observer to this game. The observer should be notified of any 
     * kind of move that occurs in this game by sending them the appropriate 
     * notification objects of <code>common.notifications</code>.
     * 
     * @param observer the observer to be added.
     */
    void addObserver(Observer observer);

    /**
     * Returns the player whose turn it is.
     *
     * @return the player whose turn it is.
     */
    PlayerColor getCurrentPlayer();

    /**
     * <p>Returns a list of fields on which the current player can introduce new
     * pieces.</p>
     * <p>The list contains {@link Tuple}s of an integer value and a
     * {@link HexDirection}. Each tuple denotes a field that is adjacent to a
     * field that already contains a piece. The integer number denotes the piece
     * (in the range {@code [0,#introduced-1]}). The second value is the
     * direction in which the field lies.</p>
     *
     * @return a list of possible fields for introduction.
     */
    ArrayList<Tuple<Integer, HexDirection>> getIntroduceable();    
    
    /**
     * Returns the total number of pieces already introduced.
     *
     * @return the total number of pieces already introduced.
     */
    int getIntroducedPieces();

    /**
     * Returns the type of piece with the specified index.
     *
     * @param pieceIndex the index of the piece.
     * @return the type of the piece.
     */
    PieceType getPieceType(int pieceIndex);

    /**
     * <p>Returns a list of fields which are reachable by a piece.</p>
     * <p>The list contains {@link Tuple}s of an integer value and a
     * {@link HexDirection}. Each tuple denotes a field that is adjacent to a
     * field that already contains a piece. The integer number denotes the piece
     * (in the range {@code [0,#introduced-1]}). The second value is the
     * direction in which the field lies.</p>
     *
     * @param pieceIndex the index of the piece.
     * @return a list of fields reachable by the piece.
     */
    ArrayList<Tuple<Integer, HexDirection>> getReachable(int pieceIndex);    
    
    /**
     * Returns the number of remaining pieces of a given type for a given
     * player.
     *
     * @param player the player which is queried.
     * @param type the piece type which is queried.
     * @return the number of remaining pieces of the specified type for the 
     * specified player.
     */
    int getRemainingPieces(PlayerColor player, PieceType type);

    /**
     * Introduces the first piece (which is of the white player, who always
     * starts the game) to the game.
     *
     * @param pieceType the type of the first piece.
     */
    void introduce(PieceType pieceType);

    /**
     * Introduces the first piece for the black player (which has to be adjacent
     * to the first (white) piece).
     *
     * @param pieceType the type of the first black piece.
     * @param hexDirection the relative position of the piece to the first white
     * piece.
     */
    void introduce(PieceType pieceType, HexDirection hexDirection);

    /**
     * <p>Introduces a piece to the game (for the current player). The position
     * is specified relatively to some other piece already introduced to the
     * game.</p>
     * <p>The anchor piece is specified by an index which is in the range of
     * {@code [0,#introduced-1]}. The direction is then specified relatively to
     * this piece.</p>
     *
     * @param pieceType the type of the piece that is introduced.
     * @param anchorPiece the index (starting with 0) of the piece used as
     * anchor.
     * @param hexDirection the relative position of the new piece to the anchor
     * piece.
     */
    void introduce(PieceType pieceType, int anchorPiece, HexDirection hexDirection);

    /**
     * Returns whether this game is over or not.
     *
     * @return {@code true} if the game is over, {@code false} otherwise.
     */
    boolean isGameOver();    
    
    /**
     * Returns whether undoing moves is possible. This is necessary for replays,
     * and can later be of use when using this model with a KI.
     * 
     * @return whether undoing moves is possible.
     */
    boolean isUndoAllowed();    
    
    /**
     * <p>Moves a piece. As for introduction the target position is specified
     * relatively to an arbitrary piece (which must be a neighbor of the new
     * position).</p>
     * <p>Both pieces are specified by its index, starting with 0.</p>
     *
     * @param piece the index of the piece that is moved
     * @param anchorPiece the index of the anchor piece
     * @param hexDirection the direction relatively to the anchor piece
     */
    void move(int piece, int anchorPiece, HexDirection hexDirection);
    
    /**
     * Reverts the last move, if it was a regular move an not an introduction. 
     * The parameters specify the move that is to be undone.
     */
    void revertMove(int piece, int anchorPiece, HexDirection hexDirection);

    /**
     * Reverts the last move, if it was the introduction of a piece onto the 
     * field. This is useful for replays and AIs.
     */
    void revertIntroduction();    
    
    /**
     * Sets whether undoing moves is allowed.
     * 
     * @param allowed whether undoing moves is allowed.
     */
    void setUndoAllowed(boolean allowed);    
}
